#!/usr/bin/php -q
<?php
#
# quick-hack script to keep comments between versions 1.2 and 2.x of Nagios

# Author: Andreas Ericsson <ae@op5.se>
#
# This script is in the public domain.
# Use at your own risk. If anything breaks, you're on your own.
#


# Translation tables
$cmt_type_table = array('HOST_COMMENT' => 'hostcomment',
						'SERVICE_COMMENT' => 'servicecomment');

$cmt_var_table = false;
$cmt_var_table['hostcomment'] =
  array(false, 'comment_id', 'host_name', 'persistent', 'author', 'comment_data');
$cmt_var_table['servicecomment'] =
  array(false, 'comment_id', 'host_name', 'service_description', 'persistent', 'author', 'comment_data');

# Variables newly introduced.
$comment_news = array('source' => 1,
					  'entry_type' => 1,
					  'expires' => 0,
					  'expire_time' => 0);


# output order for pretty-printing
$output_order =
  array('host_name', 'service_description', 'entry_type', 'comment_id',
		'source', 'persistent', 'entry_time', 'expires', 'expire_time',
		'author', 'comment_data');


# check for user stupidity
if($argc !== 2) {
	echo "Usage: $argv[0] comment.log-file\n";
	exit(1);
}

# all is set, so get down to business.
$comment_file = $argv[1];
$first = false;
$cmt_ary = false;
$i = 0;

# read and explode.
$buf = file_get_contents($comment_file);
$lines = explode("\n", $buf);

# loop the lines.
foreach($lines as $line) {
	$line = trim($line);
	# skip empty lines.
	if(!$line || !strlen($line) || $line{0} === '#') continue;

	# get the timestamp and reset the line without it.
	$line = substr($line, 1);
	$d_part = explode('] ', $line, 2);
	$when = $d_part[0];
	$line = $d_part[1];
	$tmp = explode(';', $line);
	$cmt_type = $cmt_type_table[$tmp[0]];

	# parse the exploded list. Each fields name is fetched from
	# the translation tables.
	$cmt_ary[$cmt_type][$i]['entry_time'] = $when;
	foreach($tmp as $k => $v) {
		$v_name = $cmt_var_table[$cmt_type][$k];
		if($v_name === false) continue;
		$cmt_ary[$cmt_type][$i][$v_name] = $v;
	}
	foreach($comment_news as $k => $v) {
		if(isset($cmt_ary[$cmt_type][$i][$k])) continue;
		$cmt_ary[$cmt_type][$i][$k] = $v;
	}
	$i++;
}

foreach($cmt_ary as $cmt_type => $cmt_list) {
	foreach($cmt_list as $foo => $cmt) {
		if(!$first || $first < $cmt['entry_time']) $first = $cmt['entry_time'];
		$str = explode(': ', $cmt['comment_data']);
		if(isset($str[1]) && $str[0] === 'ACKNOWLEDGEMENT') {
			$cmt['entry_type'] = 4;
			$cmt['comment_data'] = $str[1];
		}
		$cmt_list[$foo] = $cmt;
	}
	$cmt_ary[$cmt_type] = $cmt_list;
}

echo "info {\n\t$created=$first\n\tversion=2.4.0\n\t}\n\n";

foreach($cmt_ary as $cmt_type => $cmt_list) {
	foreach($cmt_list as $foo => $cmt) {
		echo "$cmt_type {\n";
		foreach($cmt as $k => $v) {
			echo "\t$k=$v\n";
		}
		echo "\t}\n\n";
	}
}

