#!/usr/bin/perl
# vim:fileencoding=utf-8

##################################################################################
# Relatório de Diagnóstico - CheckUp OpMon 1.0
##################################################################################
# 
# AUTORES: Daniel Martins & Henrique Carmo de Souza
# CRIADO EM: 17/09/2021 13:37:37
# --------------------------------------------------------------
# REVISÕES: 
# --------------------------------------------------------------
# 1.1 , 
#  - <Descrição das modificações>
# 1.2 ,                                
#  - <Descrição das modificações>
# --------------------------------------------------------------
##################################################################################
# Monta um PDF com detalhes sobre a situação do ambiente monitorado pelo OpMon
# www.opservices.com.br 
# daniel.martins@opservices.com
# henrique.carmo@opservices.com
##################################################################################

my $opmonProc = `pgrep -cfx '/usr/local/opmon/bin/opmon -d -u /usr/local/opmon/etc/opmon.cfg'`;
if ($opmonProc < 2) {
    print "OpMon not running\n";
    exit(2);
}

print("Check-Up OpMon v1.0\n\n Relatório está sendo gerado... ");
use strict; 
use warnings;
use utf8;
use IPC::System::Simple qw(system capture);
use PDF::Create;
use File::Slurp;
use POSIX qw(strftime);
use DBI;
use MIME::Lite;
use FindBin;

my $dir = $FindBin::Bin;
my $user = undef;
my $pass = undef;
my $host = undef;
my $filenamedb = '/usr/local/opmon/etc/db.php';
# <?php
#        $DBUSER="root";
#        $DBPASS="oppass";
#        $DBHOST="127.0.0.1";
# ?>
if(open(FH, '<', $filenamedb))
{
    while(<FH>)
    {
	if ($_ =~ s/DBUSER="(.*)";//) {
	    $user = $1;
	}
	if ($_ =~ s/DBPASS="(.*)";//) {
	    $pass = $1;
	}
	if ($_ =~ s/DBHOST="(.*)";//) {
	    $host = $1;
	}
    }

    close(FH);
} else {
    print "Couldn't Open a file $filenamedb\n";
    exit(2);
}

if (! $user or ! $host) {
    print "Error parsing a file $filenamedb\n";
    exit(2);
}

my $dbh = DBI->connect("DBI:mysql:opcfg:".$host,$user,$pass);

my $sql = "SELECT _company_email FROM nagios_main";
my $sth = $dbh->prepare($sql);
$sth->execute();

my $companyEmail = $sth->fetchrow_array();
$sth->finish();

$sql = "SELECT _company_name FROM nagios_main";
$sth = $dbh->prepare($sql);
$sth->execute();

my $companyName = $sth->fetchrow_array();
$sth->finish();

##################################################################################
# Criação do PDF
##################################################################################

my $newCompanyName = $companyName =~ s/\s/_/gr;
my $filename = $newCompanyName."-CheckUp-OpMon.pdf";

my $strVersion = "CheckUp OpMon v1.2";

my $pdf = PDF::Create->new(
    'filename'     => $dir.'/'.$filename,
    'Author'       => 'OpServices',
    'Title'        => 'Relatório de Diagnóstico - '.$strVersion
#    'CreationDate' => [ localtime ]
);

# Adiciona uma pagina A4
my $root = $pdf->new_page('MediaBox' => $pdf->get_page_size('A4'));
 
# Adiciona paginas que herdam atributos de $root
my $page1 = $root->new_page;
my $page2 = $root->new_page('MediaBox' => $pdf->get_page_size('A4L'));
my $page3 = $root->new_page;
my $page4 = $root->new_page;
my $page5 = $root->new_page;
my $page6 = $root->new_page;

# Definir fonte
my $font = $pdf->font('BaseFont' => 'Helvetica');
my $font2 = $pdf->font('BaseFont' => 'Helvetica-Bold');
my $font3 = $pdf->font('BaseFont' => 'Helvetica-Oblique');

# Prepara indice
$pdf->new_outline('Title' => 'Página 1', 'Destination' => $page1);
$pdf->new_outline('Title' => 'Página 2', 'Destination' => $page2);
$pdf->new_outline('Title' => 'Página 3', 'Destination' => $page3);
$pdf->new_outline('Title' => 'Página 4', 'Destination' => $page4);
$pdf->new_outline('Title' => 'Página 5', 'Destination' => $page5);
$pdf->new_outline('Title' => 'Página 6', 'Destination' => $page6);

##################################################################################
# Layout da primeira página
##################################################################################


my $date = strftime "%d/%m/%Y", localtime;
$page1->stringc($font2, 22, 306, 710, $companyName);
$page1->stringc($font2, 22, 306, 580, 'CheckUp OpMon');
$page1->stringc($font, 18, 306, 400, 'Relatório de diagnóstico - '.$date);
$page1->stringc($font3, 14, 306, 100, 'OpServices - '.$strVersion);
#$page1->stringc($font, 10, 306, 70, localtime); #substituir por CreationDate


##################################################################################
# Prepara o conteúdo para print da visão tática
##################################################################################

my $runscript_visaotatica = `node $dir/puppeteer/generate_screenshot/generate_screenshot.js`;
my $img_visaotatica = $pdf->image($dir.'/puppeteer/generate_screenshot/screenshots/visaotatica.jpg');

#$page2->stringc($font2, 32, 450, 540, "Visão Tática");
$page2->image(
    image  => $img_visaotatica,
    xscale => 0.43, # scale image for better quality
    yscale => 0.43,
    xpos   => 7,
    ypos   => 65,
    xalign => 0,
    yalign => 0,
);

##################################################################################
# Querys dos parágrafos 1, 2 e 3.
##################################################################################

#my $resultadoQuery_Paragrafo1 = `/usr/local/opmon/libexec/opservices/CheckUp_OpMon/check_problems_item1_report.pl`;
#my $resultadoQuery_Hosts = `/usr/local/opmon/libexec/opservices/CheckUp_OpMon/check_last_host_created.pl`;
#my $resultadoQuery_Services = `/usr/local/opmon/libexec/opservices/CheckUp_OpMon/check_last_created_service.pl`;
#my $resultadoQuery_Paragrafo3 = `/usr/local/opmon/libexec/opservices/CheckUp_OpMon/check_scat.pl`;
#my $resultadoQuery_Paragrafo9 = `/usr/local/opmon/libexec/opservices/CheckUp_OpMon/check_host_last_state_change.pl`;
#my $check_latency = `/usr/local/opmon/libexec/opservices/CheckUp_OpMon/check_latency.pl`;

my $resultadoQuery_Paragrafo1 = `$dir/scripts/check_problems_item1_report.pl`;
my $resultadoQuery_Hosts = `$dir/scripts/check_last_host_created.pl`;
my $resultadoQuery_Services = `$dir/scripts/check_last_created_service.pl`;
my $resultadoQuery_Paragrafo3 = `$dir/scripts/check_scat.pl`;
my $resultadoQuery_Paragrafo9 = `$dir/scripts/check_host_last_state_change.pl`;
my $check_latency = `$dir/scripts/check_latency.pl`;

##################################################################################
# Conteúdo da Segunda Página
##################################################################################

my $exibeConteudoSegundaPagina = qq{
Ao longo de mais de 20 anos de experiência com o sistema de gerenciamento de redes e sistemas, a OpServices acumulou uma vasta experiência para ajudar você a utilizar as melhores práticas de gestão em seu ambiente de TI. Um dos itens que consideramos muito importante é a correta manutenção da plataforma OpMon. E como podemos saber se o OpMon está com sua configuração ideal? Chamamos isso de Entropia do sistema, ou simplificando, o nível de desorganização do sistema. Entropia é a tendência dos sistemas cibernéticos de se desorganizar, perdendo energia e informação e rumando para a autodestruição.
Já a Sintropia é a programação dos sistemas cibernéticos para se organizar e reorganizar de modo a manter ou repor energia e informação visando a preservar sua configuração e existência, um programa de auto-preservação. A Entropia do OpMon é um auto-monitoramento feito de forma automática pela plataforma e informa a quantidade de hosts e serviços que tiveram suas checagens e/ou notificações desabilitadas, por uma razão ou outra. Pois bem, se o seu ambiente possuir um índice de mais de 3% de checagens e/ou notificações desabilitadas, baseado na experiência da OpServices, isso mostra que seu ambiente começa a dar sinais de entropia (desorganização sistêmica) e indica que você deveria olhar com mais atenção para entender os motivos pelos quais isto está acontecendo.

Talvez seja importante rever os itens que estão desabilitados e se eles não forem importantes, é muito provável que eles sejam fortes candidatos a serem removidos da monitoração. Bem como itens que estão a muitos dias em estado crítico e nenhuma ação foi tomada para o seu restabelecimento. Se você possuir algum host ou serviço em estado DOWN por mais de 30 dias, faça uma revisão do objeto e avalie a real necessidade dele estar sendo monitorado. A tendência, em qualquer sistema, é de que a entropia cresce de forma exponencial se não for devidamente controlada. Portanto, é muito importante tomar providências o mais rápido possível, ou seja, aplicar a Sintropia para manter os sistemas em ótimo estado de funcionamento.

Abaixo estão os indicadores que identificamos em seu ambiente que precisam ser trabalhados para que o OpMon forneça os indicadores e o monitoramento correto do ambiente. Estamos a disposição para ajudá-los no processo de correção e indicar as melhores práticas de gestão de sua TI.
};

##################################################################################
# Conteúdo do restante do PDF (páginas 3 em diante)
##################################################################################

my $tituloPrimeiroParagrafo = qq{
1. Problema: Excesso de incidentes
};

my $textoPrimeiroParagrafo = qq{
Por que isso pode ser um problema: em nossa experiência sabemos que ambientes com poucos incidentes são sinais de atenção aos problemas críticos. Excesso de incidentes pode significar alertas/thresholds mal configurados ou itens que não deveriam ser monitorados por não serem importantes, visto que as falhas não estão sendo corrigidas.
};

my $thresholdPrimeiroParagrafo = qq{
Threshold: Mais de 10% do total da infra.
};

my $exibeDiagnostico = qq{
Diagnóstico:
};

my $tituloSegundoParagrafo = qq{
2. Problema: Muito tempo sem adição/remoção de hosts ou serviços
};

my $textoSegundoParagrafo = qq{
Por que isso pode ser um problema: sabemos que os ambientes de nossos clientes estão cada vez mais dinâmicos e que clientes que constantemente adicionam e removem hosts possuem probabilidade maior de ter um ambiente bem gerenciado. Existe alguma forma de ajudá-los a fazer isto de forma mais automática? Ou algum problema em relação ao limite de licenças disponíveis?
};

my $thresholdSegundoParagrafo = qq{
Threshold: Mais de 2 meses sem mudança.
};

my $ultimoHost = qq{
Último host / data em que foi adicionado:
};

my $ultimoServiço = qq{
Último host e serviço / data em que foi adicionado:
};

my $resultadoPositivo_SegundoParagrafo = qq{
A data da última inserção de um Host/Serviço ao monitoramento a menos de 2 meses mostra uma administração atenta ao ambiente.
};

my $resultadoNegativo_SegundoParagrafo = qq{
A data da última inserção de um Host/Serviço ao monitoramento a mais de 2 meses indica uma necessidade de revisão atenta ao ambiente.
};

my $tituloTerceiroParagrafo = qq{
3. Problema: Não estruturar a monitoração (causa raiz)
};

my $textoTerceiroParagrafo = qq{
Por que isso pode ser um problema: O monitoramento de disponibilidade e performance é o básico e o primeiro passo em qualquer estrutura a ser monitorada. Entretanto, gera pouco valor a TI que acaba atuando de forma reativa. Boas práticas de governança sugerem a estruturação da monitoração a fim de se descobrir, por exemplo, quando a loja online está indisponível qual elemento de infraestrutura está impactando. É indicado configurar catálogos de serviços, topologia de redes e grupos de hosts e serviços. Como podemos ajudar a implementar estes catálogos?
};

my $thresholdTerceiroParagrafo = qq{
Threshold: Não ter configurados catálogos, topologia e grupo de hosts e serviços.
};

my $tituloQuartoParafrago = qq{
4. Problema: Plataforma desatualizada
};

my $textoQuartoParafrago = qq{
Por que isso pode ser um problema: A atualização da solução garante a utilização dos últimos recursos e correções de segurança. Em alguns casos é recomendada a utilização de um ambiente de homologação, principalmente em ambientes muito customizados.
};

my $resultadoPositivo_QuartoParagrafo = qq{
A plataforma está em uma versão recente, portanto uma atualização urgente não é necessária nesse momento.
};

my $resultadoNegativo_QuartoParagrafo = qq{
A plataforma não está nas versões mais recentes disponíveis e recomendamos que seja atualizada urgentemente.
};

my $thresholdQuartoParagrafo = qq{
Threshold: Verifique os Changelogs em kb.opservices.com.br/article-categories/changelogs
};

my $tituloQuintoParagrafo = qq{
5. Problema: Análise de Dashboards
};

my $textoQuintoParagrafo = qq{
Por que isso pode ser um problema: Um dos principais diferenciais da nossa plataforma são os dashboards, que permitem maior visibilidade do ambiente de TI e de negócio. Empresas que trabalham bem a representação gráfica das visões costumam valorizar sua área perante o restante da organização.
};

my $thresholdQuintoParagrafo = qq{
Threshold: Pelo menos uma nova dashboard nos últimos 2 meses.
};

my $resultadoPositivo_QuintoParagrafo = qq{
Encontramos uma quantidade significativa de dashboards em seu OpMon! Isso demonstra que a função vem sendo utilizada frequentemente, conforme o recomendado.
};

my $resultadoNegativo_QuintoParagrafo = qq{
Não encontramos nenhum dashboard criado nos últimos 2 meses, demonstrando que a função não está sendo utilizada de forma frequente, conforme o recomendado.
};

my $tituloSextoParagrafo = qq{
6. Problema: Alertas não escalonáveis
};

my $textoSextoParagrafo = qq{
Por que isso pode ser um problema: O correto envio de alertas para os responsáveis e sua posterior escalação para outros responsáveis é fundamental para que a plataforma possa enviar os alarmes para as pessoas certas e no tempo certo. Se a plataforma não possuir uma cadeia de envio de alertas devidamente configurados, seus problemas podem ficar sem solução de forma prolongada e problemas sem responsáveis tornam um pequeno incidente em uma porta para a indisponibilidade de sistemas.
};

my $thresholdSextoParagrafo = qq{
Threshold: Configuração correta de responsáveis pelos serviços monitorados.
};

my $resultadoPositivo_SextoParagrafo = qq{
Encontramos configuração de escalonamento dos alarmes do ambiente.
};

my $resultadoNegativo_SextoParagrafo = qq{
Não existe nenhuma configuração de escalonamento dos alarmes no ambiente atualmente. Recomendamos o uso de alertas escalonáveis nos principais objetos a serem monitorados para uma melhor utilização e visibilidade dos alarmes críticos no ambiente.
};


my $tituloSetimoParagrafo = qq{
7. Problema: Latência da monitoração
};

my $textoSetimoParagrafo = qq{
Por que isso pode ser um problema: A latência na monitoração informa quanto tempo a plataforma está atrasando suas checagens em função de carga excessiva. O correto dimensionamento do hardware permite que os dados coletados na monitoração tragam informações acuradas e não haja perda na confiabilidade das informações. É importante prestar atenção a latência para que possamos ajudá-los na correta configuração e dimensionamento da plataforma.
};

my $thresholdSetimoParagrafo = qq{
Threshold: Latência maior que 30 segundos nas checagens.
};

my $resultadoPositivo_SetimoParagrafo = qq{
Esse indicador nos mostra que a latência atual está bastante adequada e indica que a plataforma está operando dentro da normalidade.
};

my $resultadoNegativo_SetimoParagrafo = qq{
Esse indicador nos mostra que a latência atual não está adequada e indica que a plataforma pode não estar operando dentro da normalidade.
};

my $tituloOitavoParagrafo = qq{
8. Problema: Número de relatórios gerados
};

my $textoOitavoParagrafo = qq{
Por que isso pode ser um problema: A emissão de relatórios regulares para acompanhar a disponibilidade e a saúde do seu ambiente de TI mostra a maturidade da gestão da sua infra-estrutura. Além de apontar possíveis gargalos ou problemas, mostra que a gestão está atenta ao planejamento da sua capacidade instalada, exibindo seus recursos e sua utilização. Podemos ajudá-lo na programação do envio dos relatórios mais importantes para todos os responsáveis pela infraestrutura de TI da empresa.
};

my $thresholdOitavoParagrafo = qq{
Threshold: Quantidade/existência de relatórios agendados.
};

my $resultadoPositivo_OitavoParagrafo = qq{
Este indicador mostra que há relatórios configurados que estão sendo enviados periodicamente. Entendemos que os relatórios são importantes para a visibilidade do ambiente.
};

my $resultadoNegativo_OitavoParagrafo = qq{
Este indicador mostra há poucos relatórios que estão sendo enviados periodicamente. Entendemos que os relatórios são importantes para a visibilidade do ambiente.
};

my $tituloNonoParagrafo = qq{
9. Problema: Hosts/Serviços em estado crítico por muito tempo
};

my $textoNonoParagrafo = qq{
Por que isso pode ser um problema: Hosts/ICs/Serviços a muito tempo em estado Down/Crítico sem um devido acknowledge do problema não deveriam estar mais na monitoração. Se o objeto está a um tempo excessivo sem ser corrigido, ele deve ser removido da monitoração pois já perdeu o sentido desse objeto ser monitorado.
};

my $thresholdNonoParagrafo = qq{
Threshold: Objetos em estado de Down/Crítico a mais de 1 dia.
};

my $resultadoPositivo_NonoParagrafo = qq{
Este indicador é exibido no home dashboard do OpMon e não existem Hosts ou serviços com menos de 1 dia em estado Crítico, mostrando que o ambiente está sendo monitorado com a frequência recomendada.
};

my $resultadoNegativo_NonoParagrafo = qq{
Este indicador é exibido no home dashboard do OpMon e existem Hosts ou serviços com menos de 1 dia em estado Crítico, mostrando que o ambiente não está sendo monitorado com a frequência recomendada.
};

##################################################################################
# Leitura dos resultados
##################################################################################

my $resultado1 = read_file($dir.'/files/resultado1.txt');
my $resultado2 = read_file($dir.'/files/resultado2.txt');
my $resultado3 = read_file($dir.'/files/resultado3.txt');
my $scat_result = read_file($dir.'/files/scat-result.txt');
my $top_result = read_file($dir.'/files/top-result.txt');
my $hgroups_result = read_file($dir.'/files/hgroups-result.txt');
my $dashboards = read_file($dir.'/files/dashboards.txt');
my $opmon_version = read_file($dir.'/files/opversion.txt');
my $escalations = read_file($dir.'/files/escalations.txt');
my $scheduled_reports = read_file($dir.'/files/scheduled-reports.txt');
my $latency = read_file($dir.'/files/latency-result.txt');
my $check_hosts = read_file($dir.'/files/check_hosts_critical.txt');

##################################################################################
# Variáveis para teste das condicionais
##################################################################################

#my $resultado1_2 = 5.5; 
#my $resultado2_2 = 80; 
#my $resultado3_2 = 7; 
#my $scat_result_2 = 5;
#my $top_result_2 = 0; 
#my $hgroups_result_2 = 2;
#my $dashboards_2 = 10; 
#my $opmon_version_2 = '9.2.8'; 
#my $escalations_2 = 2; 
#my $scheduled_reports2 = '0'; 
#my $latency_2 = "71"; 
#my $latency_5 = 5;
#my ($int_latency) = $latency_2 =~ /(\d+)/;
#my $check_hosts_2 = 2; 


##################################################################################
# Conteúdo da segunda página
##################################################################################

$page3->block_text({
		page		=> $page3,
		font		=> $font,
		text		=> $exibeConteudoSegundaPagina,
		font_size	=> 12,
		line_width	=> 535,
		start_y		=> 785,
		end_y		=> 80,
		'x'			=> 30,
		'y'			=> 779,
	});

##################################################################################
# Conteúdo do primeiro parágrafo
##################################################################################

if($resultado1 > 10.0){
	$page4->setrgbcolor(255, 0, 0);
	$page4->block_text({
		page            => $page4,
		text            => $resultadoQuery_Paragrafo1,
		font	        => $font2,
		font_size       => 12,
		line_width      => 535,
		start_y         => 639,
		end_y           => 620,
		'x'             => 30,
		'y'             => 637,
	});
	$page4->block_text({
		page            => $page4,
		font            => $font2,
		text            => $tituloPrimeiroParagrafo,
		font_size       => 12,
		line_width      => 535,
		start_y         => 792,
		end_y           => 785,
		'x'             => 30,
		'y'             => 789,
	});

} else {
	$page4->setrgbcolor(0, 155, 0);
	$page4->block_text({
		page            => $page4,
		text            => $resultadoQuery_Paragrafo1,
		font	        => $font2,
		font_size       => 12,
		line_width      => 535,
		start_y         => 639,
		end_y           => 620,
		'x'             => 30,
		'y'             => 637,
	});
	$page4->block_text({
		page            => $page4,
		font            => $font2,
		text            => $tituloPrimeiroParagrafo,
		font_size       => 12,
		line_width      => 535,
		start_y         => 792,
		end_y           => 785,
		'x'             => 30,
		'y'             => 789,
	});
}

$page4->setrgbcolor(0, 0, 0);

$page4->block_text({
		page            => $page4,
		text            => $textoPrimeiroParagrafo,
		font  			=> $font,
		font_size       => 12,
		line_width      => 535,
		start_y         => 768,
		end_y           => 690,
		'x'             => 30,
		'y'             => 766,
	});
	
$page4->block_text({
		page            => $page4,
		text            => $thresholdPrimeiroParagrafo, 
		font  			=> $font3,
		font_size       => 12,
		line_width      => 535,
		start_y         => 687,
		end_y           => 682,
		'x'             => 30,
		'y'             => 685,
	});


$page4->block_text({
		page            => $page4,
		text            => $exibeDiagnostico,
		font            => $font2,
		font_size       => 12,
		line_width      => 535,
		start_y         => 667,
		end_y           => 658,
		'x'             => 30,
		'y'             => 662,
    });

##################################################################################
# Conteúdo do Segundo Parágrafo
##################################################################################

$page4->block_text({
		page            => $page4,
		text            => $textoSegundoParagrafo,
		font            => $font,
		font_size       => 12,
		line_width      => 530,
		start_y         => 572,
		end_y           => 492,
		'x'             => 30,
		'y'             => 570,
	});

$page4->block_text({
		page            => $page4,
		text            => $thresholdSegundoParagrafo, 
		font  			=> $font3,
		font_size       => 12,
		line_width      => 535,
		start_y         => 490,
		end_y           => 485,
		'x'             => 30,
		'y'             => 488,
	});

$page4->block_text({
		page            => $page4,
		text            => $exibeDiagnostico,
		font            => $font2,
		font_size       => 12,
		line_width      => 535,
		start_y         => 467,
		end_y           => 462,
		'x'             => 30,
		'y'             => 465,
	});
	
$page4->block_text({
		page            => $page4,
		text            => $ultimoHost,
		font            => $font2,
		font_size       => 12,
		line_width      => 535,
		start_y         => 442,
		end_y           => 430,
		'x'             => 30,
		'y'             => 440,
	});
	
$page4->block_text({
		page            => $page4,
		text            => $ultimoServiço,
		font            => $font2,
		font_size       => 12,
		line_width      => 535,
		start_y         => 402,
		end_y           => 390,
		'x'             => 30,
		'y'             => 400,
	});

##################################################################################
# IF PARA DIAGNOSTICO DOS HOSTS
##################################################################################
	
if ($resultado2) {
if($resultado2 >= 60){
	$page4->setrgbcolor(255, 0, 0);
	$page4->block_text({
			page            => $page4,
			text            => $resultadoQuery_Hosts,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 422,
			end_y           => 405,
			'x'             => 30,
			'y'             => 420,
	});
	} else {
	$page4->setrgbcolor(0, 255, 0);
	$page4->block_text({
			page            => $page4,
			text            => $resultadoQuery_Hosts,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 422,
			end_y           => 405,
			'x'             => 30,
			'y'             => 420,
	});

}
}

##################################################################################
# IF PARA DIAGNÓSTICO DOS SERVIÇOS
##################################################################################

if ($resultado3) {
if($resultado3 >= '60'){
	$page4->setrgbcolor(255, 0, 0);
	$page4->block_text({
			page            => $page4,
			text            => $resultadoQuery_Services,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 382,
			end_y           => 365,
			'x'             => 30,
			'y'             => 380,
	});
	} else {
	$page4->setrgbcolor(0, 255, 0);
	$page4->block_text({
			page            => $page4,
			text            => $resultadoQuery_Services,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 382,
			end_y           => 365,
			'x'             => 30,
			'y'             => 380,
	});
}
}


##################################################################################
# IF PARA DIAGNÓSTICO FINAL DO SEGUNDO PARÁGRAFO
##################################################################################

if ($resultado2 and $resultado3) {
if($resultado2 > '60' and $resultado3 > '60') {
	$page4->setrgbcolor(255, 0, 0);
	$page4->block_text({
			page            => $page4,
			text            => $tituloSegundoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 597,
			end_y           => 590,
			'x'             => 30,
			'y'             => 595,
	});
	$page4->block_text({
			page            => $page4,
			text            => $resultadoNegativo_SegundoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 352,
			end_y           => 280,
			'x'             => 30,
			'y'             => 350,
	});	
} elsif ($resultado2 <= '60' and $resultado3 <= '60') {
	$page4->setrgbcolor(0, 255, 0);
	$page4->block_text({
			page            => $page4,
			text            => $tituloSegundoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 597,
			end_y           => 590,
			'x'             => 30,
			'y'             => 595,
	});
	$page4->block_text({
			page            => $page4,
			text            => $resultadoPositivo_SegundoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 352,
			end_y           => 280,
			'x'             => 30,
			'y'             => 350,
	});	
} else {
	$page4->setrgbcolor(0, 0, 156);
	$page4->block_text({
			page            => $page4,
			text            => $tituloSegundoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 597,
			end_y           => 590,
			'x'             => 30,
			'y'             => 595,
	});
	$page4->block_text({
			page            => $page4,
			text            => $resultadoNegativo_SegundoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 352,
			end_y           => 280,
			'x'             => 30,
			'y'             => 350,
	});	
} 
}

##################################################################################
# Conteúdo do Terceiro Parágrafo
##################################################################################
$page4->setrgbcolor(0, 0, 0);
$page4->block_text({
		page            => $page4,
		text            => $textoTerceiroParagrafo,
		font            => $font,
		font_size       => 12,
		line_width      => 535,
		start_y         => 269,
		end_y           => 140,
		'x'             => 30,
		'y'             => 267,
	});
	
$page4->block_text({
		page            => $page4,
		text            => $thresholdTerceiroParagrafo,
		font            => $font3,
		font_size       => 12,
		line_width      => 535,
		start_y         => 149,
		end_y           => 137,
		'x'             => 30,
		'y'             => 147,
	});

$page4->block_text({
		page            => $page4,
		text            => $exibeDiagnostico,
		font            => $font2,
		font_size       => 12,
		line_width      => 535,
		start_y         => 122,
		end_y           => 115,
		'x'             => 30,
		'y'             => 120,
	});

##################################################################################
# IF PARA DIAGNÓSTICO DO TERCEIRO PARÁGRAFO
##################################################################################

if (($scat_result == 0) and ($top_result == 0) and ($hgroups_result == 0)){
	$page4->setrgbcolor(255, 0, 0);
	$page4->block_text({
			page            => $page4,
			text            => $tituloTerceiroParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 538,
			start_y         => 292,
			end_y           => 278,
			'x'             => 30,
			'y'             => 290,
	});
	$page4->block_text({
			page            => $page4,
			text            => $resultadoQuery_Paragrafo3,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 92,
			end_y           => 20,
			'x'             => 30,
			'y'             => 90,
	});
	} elsif (($scat_result > 0) and ($top_result > 0) and ($hgroups_result > 0)) {
	$page4->setrgbcolor(0, 255, 0);
	$page4->block_text({
			page            => $page4,
			text            => $tituloTerceiroParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 538,
			start_y         => 292,
			end_y           => 278,
			'x'             => 30,
			'y'             => 290,
	});
	$page4->block_text({
			page            => $page4,
			text            => $resultadoQuery_Paragrafo3,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 92,
			end_y           => 20,
			'x'             => 30,
			'y'             => 90,
	});
	
} else {
	$page4->setrgbcolor(0, 0, 156);
	$page4->block_text({
			page            => $page4,
			text            => $tituloTerceiroParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 538,
			start_y         => 292,
			end_y           => 278,
			'x'             => 30,
			'y'             => 290,
	});
	$page4->block_text({
			page            => $page4,
			text            => $resultadoQuery_Paragrafo3,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 92,
			end_y           => 20,
			'x'             => 30,
			'y'             => 90,
	});	
}


$page5->block_text({
		page            => $page5,
		text            => $textoQuartoParafrago,
		font            => $font,
		font_size       => 12,
		line_width      => 535,
		start_y         => 778,
		end_y           => 712,
		'x'             => 30,
		'y'             => 774,
	});

$page5->block_text({
		page            => $page5,
		text            => $thresholdQuartoParagrafo,
		font            => $font3,
		font_size       => 12,
		line_width      => 535,
		start_y         => 712,
		end_y           => 700,
		'x'             => 30,
		'y'             => 710,
	});

$page5->block_text({
		page            => $page5,
		text            => $exibeDiagnostico,
		font            => $font2,
		font_size       => 12,
		line_width      => 535,
		start_y         => 682,
		end_y           => 677,
		'x'             => 30,
		'y'             => 680,
	});
	
##################################################################################
# IF PARA DIAGNÓSTICO DO QUARTO PARÁGRAFO
##################################################################################

my ($int_opversion) = $opmon_version =~ /(\d+)/; 
#my ($int_opversion2) = $opmon_version_2 =~ /(\d+)/g; 

if($int_opversion == '10'){
	$page5->setrgbcolor(0, 255, 0);
	$page5->block_text({
			page            => $page5,
			text            => $resultadoPositivo_QuartoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 652,
			end_y           => 625,
			'x'             => 30,
			'y'             => 650,
	});
	$page5->block_text({
			page            => $page5,
			text            => $tituloQuartoParafrago,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 799,
			end_y           => 787,
			'x'             => 30,
			'y'             => 797,
	});
} elsif ($int_opversion == '9') {
	$page5->setrgbcolor(255, 0, 0);
	$page5->block_text({
			page            => $page5,
			text            => $resultadoNegativo_QuartoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 652,
			end_y           => 625,
			'x'             => 30,
			'y'             => 650,
	});
		$page5->block_text({
			page            => $page5,
			text            => $tituloQuartoParafrago,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 799,
			end_y           => 787,
			'x'             => 30,
			'y'             => 797,
	});
} elsif ($int_opversion == '8') {
	$page5->setrgbcolor(255, 0, 0);
	$page5->block_text({
			page            => $page5,
			text            => $resultadoPositivo_QuartoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 652,
			end_y           => 625,
			'x'             => 30,
			'y'             => 650,
	});
		$page5->block_text({
			page            => $page5,
			text            => $tituloQuartoParafrago,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 799,
			end_y           => 787,
			'x'             => 30,
			'y'             => 797,
	});
}  elsif ($int_opversion == '7') {
	$page5->setrgbcolor(255, 0, 0);
	$page5->block_text({
			page            => $page5,
			text            => $resultadoPositivo_QuartoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 652,
			end_y           => 625,
			'x'             => 30,
			'y'             => 650,
	});
		$page5->block_text({
			page            => $page5,
			text            => $tituloQuartoParafrago,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 799,
			end_y           => 787,
			'x'             => 30,
			'y'             => 797,
	});
}

$page5->setrgbcolor(0, 0, 0);
$page5->block_text({
		page            => $page5,
		text            => $textoQuintoParagrafo,
		font            => $font,
		font_size       => 12,
		line_width      => 535,
		start_y         => 566,
		end_y           => 488,
		'x'             => 30,
		'y'             => 564,
	});

$page5->block_text({
		page            => $page5,
		text            => $thresholdQuintoParagrafo,
		font            => $font3,
		font_size       => 12,
		line_width      => 535,
		start_y         => 486,
		end_y           => 480,
		'x'             => 30,
		'y'             => 484,
	});

$page5->block_text({
		page            => $page5,
		text            => $exibeDiagnostico,
		font            => $font2,
		font_size       => 12,
		line_width      => 503,
		start_y         => 460,
		end_y           => 453,
		'x'             => 30,
		'y'             => 457,
	});
	
##################################################################################
# IF PARA DIAGNÓSTICO DO QUINTO PARÁGRAFO
##################################################################################
	
if($dashboards >= '1'){
	$page5->setrgbcolor(0, 255, 0);
	$page5->block_text({
			page            => $page5,
			text            => $resultadoPositivo_QuintoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 432,
			end_y           => 400,
			'x'             => 30,
			'y'             => 430,
	});
	$page5->block_text({
			page            => $page5,
			text            => $tituloQuintoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 590,
			end_y           => 582,
			'x'             => 30,
			'y'             => 585,
	});
	} else {
	$page5->setrgbcolor(255, 0, 0);
	$page5->block_text({
			page            => $page5,
			text            => $resultadoNegativo_QuintoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 432,
			end_y           => 400,
			'x'             => 30,
			'y'             => 430,
	});
		$page5->block_text({
			page            => $page5,
			text            => $tituloQuintoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 590,
			end_y           => 582,
			'x'             => 30,
			'y'             => 585,
	});

}

$page5->setrgbcolor(0, 0, 0);
$page5->block_text({
		page            => $page5,
		text            => $textoSextoParagrafo,
		font            => $font,
		font_size       => 12,
		line_width      => 535,
		start_y         => 344,
		end_y           => 224,
		'x'             => 30,
		'y'             => 342,
	});

$page5->block_text({
		page            => $page5,
		text            => $thresholdSextoParagrafo,
		font            => $font3,
		font_size       => 12,
		line_width      => 535,
		start_y         => 222,
		end_y           => 210,
		'x'             => 30,
		'y'             => 220,
	});

$page5->block_text({
		page            => $page5,
		text            => $exibeDiagnostico,
		font            => $font2,
		font_size       => 12,
		line_width      => 535,
		start_y         => 195,
		end_y           => 188,
		'x'             => 30,
		'y'             => 192,
	});
	
##################################################################################
# IF PARA DIAGNÓSTICO DO SEXTO PARÁGRAFO
##################################################################################
	
if($escalations == '0'){
	$page5->setrgbcolor(255, 0, 0);
	$page5->block_text({
			page            => $page5,
			text            => $resultadoNegativo_SextoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 167,
			end_y           => 20,
			'x'             => 30,
			'y'             => 165,
	});
	$page5->block_text({
			page            => $page5,
			text            => $tituloSextoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 367,
			end_y           => 362,
			'x'             => 30,
			'y'             => 365,
	});
	} else {
	$page5->setrgbcolor(0, 255, 0);
	$page5->block_text({
			page            => $page5,
			text            => $resultadoPositivo_SextoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 167,
			end_y           => 20,
			'x'             => 30,
			'y'             => 165,
	});
		$page5->block_text({
			page            => $page5,
			text            => $tituloSextoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 367,
			end_y           => 362,
			'x'             => 30,
			'y'             => 365,
	});
}

$page6->block_text({
		page            => $page6,
		text            => $textoSetimoParagrafo,
		font            => $font,
		font_size       => 12,
		line_width      => 535,
		start_y         => 766,
		end_y           => 664,
		'x'             => 30,
		'y'             => 764,
	});
	
$page6->block_text({
		page            => $page6,
		text            => $thresholdSetimoParagrafo,
		font            => $font3,
		font_size       => 12,
		line_width      => 535,
		start_y         => 662,
		end_y           => 650,
		'x'             => 30,
		'y'             => 660,
	});

$page6->block_text({
		page            => $page6,
		text            => $exibeDiagnostico,
		font            => $font2,
		font_size       => 12,
		line_width      => 535,
		start_y         => 634,
		end_y           => 625,
		'x'             => 30,
		'y'             => 632,
	});


##################################################################################
# IF PARA DIAGNÓSTICO DO SETIMO PARÁGRAFO
##################################################################################
	
if($latency <= '5'){
	$page6->setrgbcolor(0, 255, 0);
	$page6->block_text({
			page            => $page6,
			text            => $tituloSetimoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 791,
			end_y           => 785,
			'x'             => 30,
			'y'             => 789,
	});
	$page6->block_text({
			page            => $page6,
			text            => $resultadoPositivo_SetimoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 607,
			end_y           => 562,
			'x'             => 30,
			'y'             => 605,
	});
	} else {
	$page6->setrgbcolor(255, 0, 0);
	$page6->block_text({
			page            => $page6,
			text            => $tituloSetimoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 791,
			end_y           => 785,
			'x'             => 30,
			'y'             => 789,
	});
	$page6->block_text({
			page            => $page6,
			text            => $resultadoNegativo_SetimoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 607,
			end_y           => 562,
			'x'             => 30,
			'y'             => 605,
	});

}

$page6->setrgbcolor(0, 0, 0);
$page6->block_text({
		page            => $page6,
		text            => $textoOitavoParagrafo,
		font            => $font,
		font_size       => 12,
		line_width      => 535,
		start_y         => 522,
		end_y           => 410,
		'x'             => 30,
		'y'             => 520,
	});
	
$page6->block_text({
		page            => $page6,
		text            => $thresholdOitavoParagrafo,
		font            => $font3,
		font_size       => 12,
		line_width      => 535,
		start_y         => 400,
		end_y           => 395,
		'x'             => 30,
		'y'             => 398,
	});

$page6->block_text({
		page            => $page6,
		text            => $exibeDiagnostico,
		font            => $font2,
		font_size       => 12,
		line_width      => 535,
		start_y         => 372,
		end_y           => 367,
		'x'             => 30,
		'y'             => 370,
	});

##################################################################################
# IF PARA DIAGNÓSTICO DO OITAVO PARÁGRAFO
##################################################################################
	
if($scheduled_reports == '0') {
	$page6->setrgbcolor(255, 0, 0);
	$page6->block_text({
			page            => $page6,
			text            => $tituloOitavoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 547,
			end_y           => 542,
			'x'             => 30,
			'y'             => 545,
	});
	$page6->block_text({
			page            => $page6,
			text            => $resultadoNegativo_OitavoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 342,
			end_y           => 300,
			'x'             => 30,
			'y'             => 340,
	});
	} else {
	$page6->setrgbcolor(0, 255, 0);
		$page6->block_text({
			page            => $page6,
			text            => $tituloOitavoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 547,
			end_y           => 542,
			'x'             => 30,
			'y'             => 545,
	});
	
	$page6->block_text({
			page            => $page6,
			text            => $resultadoPositivo_OitavoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 342,
			end_y           => 300,
			'x'             => 30,
			'y'             => 340,
	});
}

$page6->setrgbcolor(0, 0, 0);
$page6->block_text({
		page            => $page6,
		text            => $textoNonoParagrafo,
		font            => $font,
		font_size       => 12,
		line_width      => 535,
		start_y         => 252,
		end_y           => 170,
		'x'             => 30,
		'y'             => 250,
	});

$page6->block_text({
		page            => $page6,
		text            => $thresholdNonoParagrafo,
		font            => $font3,
		font_size       => 12,
		line_width      => 535,
		start_y         => 167,
		end_y           => 163,
		'x'             => 30,
		'y'             => 165,
	});

$page6->block_text({
		page            => $page6,
		text            => $exibeDiagnostico,
		font            => $font2,
		font_size       => 12,
		line_width      => 535,
		start_y         => 137,
		end_y           => 130,
		'x'             => 30,
		'y'             => 135,
	});

##################################################################################
# IF PARA DIAGNÓSTICO DO NONO PARÁGRAFO
##################################################################################
	
if($check_hosts == '0'){
	$page6->setrgbcolor(0, 255, 0);
	$page6->block_text({
			page            => $page6,
			text            => $tituloNonoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 277,
			end_y           => 272,
			'x'             => 30,
			'y'             => 275,
	});
	$page6->block_text({
			page            => $page6,
			text            => $resultadoPositivo_NonoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 107,
			end_y           => 30,
			'x'             => 30,
			'y'             => 105,
	});
	} else {
	$page6->setrgbcolor(255, 0, 0);
	$page6->block_text({
			page            => $page6,
			text            => $tituloNonoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 277,
			end_y           => 272,
			'x'             => 30,
			'y'             => 275,
	});
	$page6->block_text({
			page            => $page6,
			text            => $resultadoNegativo_NonoParagrafo,
			font            => $font2,
			font_size       => 12,
			line_width      => 535,
			start_y         => 107,
			end_y           => 30,
			'x'             => 30,
			'y'             => 105,
	});

}

print("PDF foi gerado com sucesso!\n\n");

$pdf->close;

my $to = $companyEmail;
my $from = 'opmonreport@opservices.com';

my $monthAndYear = strftime "%m/%Y", localtime;
my $subject = '['.$companyName.'] Relatório OpMon Checkup - ['.$monthAndYear.']';
my $message = 'Prezado cliente, você está recebendo um Relatório de Checkup do OpMon do mês de ['.$monthAndYear.'].
É muito importante que você analise o relatório e compartilhe o mesmo com seus colegas pois o objetivo deste relatório é ajudá-lo a melhorar a “saúde” do seu ambiente monitorado pela plataforma OpMon.
Estamos a sua disposição para ajudá-lo a interpretar o relatório e implementar as melhorias necessárias na plataforma e assim garantir o melhor uso do software.

Atenciosamente,

OpServices Tecnologia da Informação

suporte@opservices.com
';

sendEmail($from, $to, $subject, $message, $dir);
sendEmail($from, $from, $subject, $message, $dir);

sub sendEmail {
  my $from = shift;
  my $to = shift;
  my $subject = shift;
  my $message = shift;
  my $dir = shift;

  my $msg = MIME::Lite->new(
    From     => $from,
    To       => $to,
    Subject  => $subject,
    Type     => 'multipart/mixed'
  );

  $msg->attach(
    Type     => 'text/plain; charset=UTF-8',
    Data     => $message
  );


  $msg->attach(
    Type        => 'application/pdf',
    Path        => $dir."/".$filename,
    Filename    => $filename,
    Disposition => 'attachment'
  );
  $msg->send;
}

print("PDF foi enviado para $companyEmail!\n\n");
