#!/usr/bin/perl
use strict;
use warnings;
use utf8;
use DBI;
use FindBin;

my $dir = $FindBin::Bin;

my $user = undef;
my $pass = undef;
my $host = undef;
my $filename = '/usr/local/opmon/etc/db.php';
# <?php
#        $DBUSER="root";
#        $DBPASS="oppass";
#        $DBHOST="127.0.0.1";
# ?>
if(open(FH, '<', $filename))
{
    while(<FH>)
    {
	if ($_ =~ s/DBUSER="(.*)";//) {
	    $user = $1;
	}
	if ($_ =~ s/DBPASS="(.*)";//) {
	    $pass = $1;
	}
	if ($_ =~ s/DBHOST="(.*)";//) {
	    $host = $1;
	}
    }

    close(FH);
} else {
    print "Couldn't Open a file $filename\n";
    exit(2);
}

if (! $user or ! $host) {
    print "Error parsing a file $filename\n";
    exit(2);
}

my $dbh = DBI->connect("DBI:mysql:opcfg:".$host,$user,$pass);

die "failed to connect to MySQL database:DBI->errstr()" unless($dbh);

# prepare SQL statement
my $scat = $dbh->prepare("SELECT\n".
"	count(*) \n".
"FROM\n".
"	opmon4.scats where id > 1") or die "prepare statement failed: $dbh->errstr()";

$scat->execute() or warn "execution failed: $dbh->errstr()";

# prepare SQL statement
my $top = $dbh->prepare("SELECT\n".
"	count(*) \n".
"FROM\n".
"	opcfg.topology\n".
"	") or die "prepare statement failed: $dbh->errstr()";

$top->execute() or warn "execution failed: $dbh->errstr()";

# prepare SQL statement
my $hgroups = $dbh->prepare("SELECT\n".
"	count(*) \n".
"FROM\n".
"	opcfg.nagios_hostgroups \n".
"WHERE\n".
"	hostgroup_id > 10") or die "prepare statement failed: $dbh->errstr()";

$hgroups->execute() or warn "execution failed: $dbh->errstr()";

# prepare SQL statement
my $opversion = $dbh->prepare("SELECT".
"	current_version ".
"FROM".
"	opmon4.version_checker") or die "prepare statement failed: $dbh->errstr()";

$opversion->execute() or warn "execution failed: $dbh->errstr()";

# prepare SQL statement
my $dash = $dbh->prepare("SELECT\n".
"	count(*) \n".
"FROM\n".
"	grafana.dashboard \n".
"WHERE\n".
"	updated BETWEEN DATE_SUB( NOW(), INTERVAL 2 MONTH ) \n".
"	AND NOW();") or die "prepare statement failed: $dbh->errstr()";

$dash->execute() or warn "execution failed: $dbh->errstr()";

# prepare SQL statement
my $escalations = $dbh->prepare("SELECT\n".
"	count(*) \n".
"FROM\n".
"	`opcfg`.`nagios_escalations` \n") or die "prepare statement failed: $dbh->errstr()";

$escalations->execute() or warn "execution failed: $dbh->errstr()";

# prepare SQL statement
my $scheduled_reports = $dbh->prepare("SELECT\n".
"	count(*) \n".
"FROM\n".
"	`opmon4`.`scheduled_reports` \n") or die "prepare statement failed: $dbh->errstr()";

$scheduled_reports->execute() or warn "execution failed: $dbh->errstr()";

my $dash_result = $dash->fetchrow();
my $scat_result = $scat->fetchrow();
my $top_result = $top->fetchrow();
my $hgroups_result = $hgroups->fetchrow();
my $opversion_result = $opversion->fetchrow();
my $escalations_result = $escalations->fetchrow();
my $scheduled_reports_result = $scheduled_reports->fetchrow();

#if ($dash_result > 0) {
#  print("Dashboards atualizadas!");
#} else {
#  print("Dashboards desatualizadas!");
#}

if ($scat_result == 0) {
  print("Não existem catalogos configurados, ");
} else {
  print ("Catalogos configurados, ");
}

if ($top_result == 0) {
  print("não existem topologias configuradas ");
} else {
  print ("topologias configuradas ");
}

if ($hgroups_result == 0) {
  print("e não foram criados novos host groups.\n");
} else {
  print ("e host groups estão configurados.\n");
}

#if ($opversion_result lt '10') {
#  print (", versão obsoleta, ");
#}

#if ($escalations_result == 0) {
#  print("não existem escalacoes configuradas e ");
#} else {
#  print ("escalacoes configuradas e ");
#}

#if ($scheduled_reports_result == 0) {
#  print("não existem relatorios agendados.\n");
#} else {
#  print ("relatorios agendados estao configurados.\n");
#}

######################################################

#Salva conteudo numerico em arquivo TXT para consulta

my $txt1 = <<END;
$scat_result
END

my $filetxt = $dir.'/../files/scat-result.txt';

open(FH, '>', $filetxt) or die $!;
print FH $txt1;

close(FH);

######################################################

my $txt2 = <<END;
$top_result
END

my $filetxt2 = $dir.'/../files/top-result.txt';

open(FH, '>', $filetxt2) or die $!;
print FH $txt2;

close(FH);

######################################################

my $txt3 = <<END;
$hgroups_result
END

my $filetxt3 = $dir.'/../files/hgroups-result.txt';

open(FH, '>', $filetxt3) or die $!;
print FH $txt3;

close(FH);

######################################################

my $txt4 = <<END;
$dash_result
END

my $filetxt4 = $dir.'/../files/dashboards.txt';

open(FH, '>', $filetxt4) or die $!;
print FH $txt4;

close(FH);

######################################################

my $txt5 = <<END;
$opversion_result
END

my $filetxt5 = $dir.'/../files/opversion.txt';

open(FH, '>', $filetxt5) or die $!;
print FH $txt5;

close(FH);

######################################################

my $txt6 = <<END;
$escalations_result
END

my $filetxt6 = $dir.'/../files/escalations.txt';

open(FH, '>', $filetxt6) or die $!;
print FH $txt6;

close(FH);

######################################################

my $txt7 = <<END;
$scheduled_reports_result
END

my $filetxt7 = $dir.'/../files/scheduled-reports.txt';

open(FH, '>', $filetxt7) or die $!;
print FH $txt7;

close(FH);

######################################################



$scheduled_reports->finish();
$escalations->finish();
$dash->finish();
$opversion->finish();
$scat->finish();
$top->finish();
$hgroups->finish();
$dbh->disconnect();
