#!/bin/sh

PKGDIR="/var/local/src/opmon35"
PKGDIRCENTOS="$PKGDIR/centos"
PKGDIRFEDORA="$PKGDIR/fedora"
SCRIPTDIR="/usr/local/src/autoupdate"
LOGFILE="$SCRIPTDIR/log/autoupdate.log"
DEBUGFILE="$SCRIPTDIR/log/autoupdate.debug"
TMPSCRIPT="/tmp/autoupdate.tmp"
OPMONSDB="/tmp/opmons.db"
MYSQLHOST=192.168.10.1
MYSQLUSER=root
MYSQLPASS=oppass
SSHUSER=opuser
SSHPASS=3nt3rth341?97dr4g0n

# Global vars
CENTOS=""
FEDORA=""
PKGS=""

main ()
{
	RETCODE=0
	echo "Starting autoupdate.sh" >$LOGFILE

	# prepare and show an opmon select screen
	servers_screen
	INSTALLON=`/bin/sh $TMPSCRIPT`

	# select only the CentOS server due compiled packages
	osscreen $INSTALLON
	CENTOS=`/bin/sh $TMPSCRIPT`

	# get the fedora core servers
	FEDORA=""
	for F in $INSTALLON
	do
		FOUND=0
		for C in $CENTOS
		do
			if [ $F = $C ]
			then
				FOUND=1
			fi
		done

		if [ $FOUND -eq 0  ]
		then
			FEDORA="$FEDORA $F"
		fi

	done

	# prepare and show an rpm select screen
	rpm_screen
	PKGS=`/bin/sh $TMPSCRIPT`
	
	# show a message to user see what was selected
	# before continue
	MSG="SELECTED SERVERS:\n\nFedora Servers:\n"
	for S in $FEDORA
	do
		MSG="$MSG $S\n"
	done

	MSG=$MSG"\n\nCentOS Servers:\n"
	for S in $CENTOS
	do
		MSG="$MSG $S\n"
	done

	dialog --title "Confirm" --msgbox "$MSG" 20 70

        MSG="Packages:\n"
        for S in $PKGS
        do
                MSG="$MSG $S\n"
        done	

	dialog --title "Confirm" --msgbox "$MSG" 20 70

	# upload the rpm files
	upload_files 
	# show upload log file
	MSG=`cat $LOGFILE`
	dialog --title "Upload log" --msgbox "$MSG" 100 100

	install_rpms
	MSG=`cat $LOGFILE`
	dialog --title "Final log" --msgbox "$MSG" 100 100

	exit $RETCODE
}

#
# this functions uses an expect script to install 
# rpm files 
#
install_rpms ()
{

	for P in $PKGS
	do
		P=`echo $P |tr -d "\""`

		echo "Installing $P" >>$LOGFILE

		for S in $CENTOS $FEDORA
		do
			S=`echo $S |tr -d "\""`

			dialog						\
				--title 'Please wait'			\
				--infobox "\nInstalling $P in $S"	\
				10 90

			RET=`$SCRIPTDIR/install.exp $S $PORT 		\
				$SSHPASS  $P >> $DEBUGFILE`

			if [ $? = 0 ]
			then
				echo "[OK]" >>$LOGFILE
				dialog					\
					--title 'Install Status'	\
					--infobox "SUCESS"		\
					10 90
			else
				dialog					\
					--title 'Install Status'	\
					--infobox "FAILED"		\
					10 90
				echo "[FAILED]" >>$LOGFILE
			fi
		done
	done

}


#
# this functions uses an expect script to upload
# rpm files to servers
#
upload_files ()
{

	for P in $PKGS
	do
		echo "Uploading $P" >>$LOGFILE

		P=`echo $P |tr -d "\""`
		for S in $CENTOS
		do
			echo -n "> $S" >>$LOGFILE

			S=`echo $S |tr -d "\""`
			dialog						\
				--title 'Please wait'			\
				--infobox "\nUploading $P to $S"	\
				10 90

			PORT=`grep $S $OPMONSDB | 			\
				cut -d, -f3 |xargs echo -n`


			if [ -f "$PKGDIR/$P" ] 
			then
				FILE="$PKGDIR/$P"
			else
				FILE="$PKGDIRCENTOS/$P"
			fi


			RET=`$SCRIPTDIR/upload.exp $S $PORT 		\
				$SSHPASS  $FILE >> $DEBUGFILE`

			if [ $? = 0 ]
			then
				echo "[OK]" >>$LOGFILE
				dialog					\
					--title 'Upload Status'		\
					--infobox "SUCESS"		\
					10 90
			else
				dialog					\
					--title 'Upload Status'		\
					--infobox "FAILED"		\
					10 90
				echo "[FAILED]" >>$LOGFILE
			fi
			sleep 1
		done

		for S in $FEDORA
		do
			echo -n "> $S" >>$LOGFILE

			S=`echo $S |tr -d "\""`
			dialog						\
				--title 'Please wait'			\
				--infobox "\nUploading $P to $S"	\
				10 90

			PORT=`grep $S $OPMONSDB | 			\
				cut -d, -f3 |xargs echo -n`


			if [ -f "$PKGDIR/$P" ] 
			then
				FILE="$PKGDIR/$P"
			else
				FILE="$PKGDIRFEDORA/$P"
			fi

			RET=`$SCRIPTDIR/upload.exp $S $PORT 		\
				$SSHPASS  $FILE >> $DEBUGFILE`

			if [ $? = 0 ]
			then
				echo "[OK]" >>$LOGFILE
				dialog					\
					--title 'Upload Status'		\
					--infobox "SUCESS"		\
					10 90
			else
				echo "[FAILED]" >>$LOGFILE
				dialog					\
					--title 'Upload Status'		\
					--infobox "FAILED"		\
					10 90
			fi
			sleep 1

		done
	done

}


#
# shows a screen where the users selects the
# operating system of every server
#
osscreen ()
{

	echo "#!/bin/sh " >$TMPSCRIPT
	echo "dialog    --stdout \\" >>$TMPSCRIPT
	echo "  --title 'Operating System' \\" >> $TMPSCRIPT
	echo "  --checklist 'Select CentOS servers:' \\" >> $TMPSCRIPT
	echo "  0 0 0 \\" >>$TMPSCRIPT

	for I in $@
	do
		echo "  $I ' ' off \\" >>$TMPSCRIPT
	done

}

#
# prints a screen where the user selects the packages(rpms) to
# be updated on server
#
rpm_screen ()
{
	echo "#!/bin/sh " >$TMPSCRIPT
	echo "dialog 	--stdout \\" >>$TMPSCRIPT
	echo "	--title 'Packages' \\" >> $TMPSCRIPT
	echo "	--checklist 'Select packages:' \\" >> $TMPSCRIPT
	echo "	0 0 0 \\" >>$TMPSCRIPT

	for I in `ls -1 $PKGDIR $PKGDIRCENTOS $PKGDIRFEDORA |grep rpm | sort |uniq`
	do
		echo "	'$I' ' ' off \\" >>$TMPSCRIPT
	done
	
}

#
# get the servers using a mysql query stored ate getopmon.sql
# and prints a dialog box where the user must select the needed
#
servers_screen ()
{


	OPMONS=$(mysql 							\
			-u $MYSQLUSER 					\
			--password=$MYSQLPASS 				\
			-h $MYSQLHOST 					\
			opnetdb < $SCRIPTDIR/getopmons.sql)

	rm -rf $OPMONSDB
	I=0
	for OPMON in $OPMONS
	do
		# ignore the first line(mysql table header)
		if [ $I -gt 3 ]
		then

			# default ssh port
			if [ $OPMON = 'NULL' ]
			then
				OPMON=54978
			fi

			echo -n $OPMON, >>$OPMONSDB

			# the query always returns 3 as 
			# the last valid field
			if [ $OPMON = '3' ]
			then
				echo " " >>$OPMONSDB
			fi
		fi
		let I++
	done

        echo "#!/bin/sh " >$TMPSCRIPT
        echo "dialog    --stdout \\" >>$TMPSCRIPT
        echo "  --title 'OpMons' \\" >> $TMPSCRIPT
        echo "  --checklist 'Select OpMons:' \\" >> $TMPSCRIPT
        echo "  0 0 0 \\" >>$TMPSCRIPT

        for I in `cat $OPMONSDB |cut -d, -f1`
        do
                echo "  '$I' ' ' off \\" >>$TMPSCRIPT
        done

}

main
