<?php

include("/usr/local/opmon/etc/config.php");
require_once("/usr/local/opmon/share/onc/libs/autoload.php");

use Database\Driver\DefaultDB as DefaultDB;
use Prediction\Capacity as Predict;
use Livestatus\Connection\ConnectionProvider as Livestatus;
use NotificationCenter\Prediction\MessagePrediction as BadgePredict;


if (is_nagios_running() !== true) {
    print date('Y-m-d H:i:s')." - OpMon is not running".PHP_EOL;
    exit(0);
}

$query = "
    SELECT
     service_id,
     host_id,
     metric_id
    FROM
     service_metric
";

$db = new DefaultDB('opcfg');
$db->prepare($query);
$db->execute();
$elements = $db->fetchAll(DefaultDB::FETCH_OBJ);

$bursts = array();
foreach ($elements as $element) {
    print date('Y-m-d H:i:s')." - Run prediciton for H:{$element->host_id}"
        ." S:{$element->service_id} M:{$element->metric_id} ... ";

    try {
        $predict = new Predict(
            $element->metric_id,
            $element->service_id,
            $element->host_id
        );
        $predict->runBackgroundPrediction();
        $bursts[] = $predict;
        unset($predict);
        print "Burst capacity found!";
    } catch (Exception $e) {
        print $e->getMessage();
    }
    print PHP_EOL;
}

print date('Y-m-d H:i:s')." - Total of ".count($elements)." elements analyzed.".PHP_EOL;
print date('Y-m-d H:i:s')." - Total of ".count($bursts)." predictions done.".PHP_EOL;

if (count($bursts) > 0) {
    $badgePredict = new BadgePredict(count($bursts));
    $badgePredict->save();
}

print date('Y-m-d H:i:s')." - Cleaning data of from the last 8 hours...".PHP_EOL;

clean();
print "Done.".PHP_EOL;

if (Predict::getNotify() === false) {
    print date('Y-m-d H:i:s')." - Notification disabled.".PHP_EOL;
    exit(0);
}

print date('Y-m-d H:i:s')." - Processing send mails and ONC".PHP_EOL;

$count = 0;
foreach ($bursts as $burst) {
    $rDays = (int) round((strtotime($burst->getBurstDatetime()) - time()) / 60 / 60 / 24);
    print date('Y-m-d H:i:s')." - Verify burst in {$rDays} days".PHP_EOL;
    if ($rDays <= 7) {
        $svc = get_service_by_id($burst->getHostId(), $burst->getServiceId());
        $conn = Livestatus::factory("tcp");
        $query = $conn->from('services');
        $query->select('contacts');
        $query->filterBy(
            $conn->createFilter(
                array(
                    array(
                        'host_name',
                        $conn->createOperator('='),
                        $svc->get_host_name()
                    ),
                    array(
                        'description',
                        $conn->createOperator('='),
                        $svc->get_service_description()
                    )
                ),
                $conn->createOperator('And')
            )
        );
        $objects = $conn->execute()->fetchAll();
        foreach ($objects[0]->contacts as $name) {
            $contact = get_contact_by_name($name);

            $hasValidEmail = preg_match(
                '/[a-zA-Z0-9_\-\.]+@[a-zA-Z0-9_\-\.]+/',
                $contact->get_email()
            );

            if (! $hasValidEmail) {
                print date('Y-m-d H:i:s')." - Skip. Invalid email address ({$contact->get_email()}).".PHP_EOL;
                continue;
            }

			$data = new StdClass();
			$data->burstValue = $burst->getBurstValue();
			$data->burstDatetime = $burst->getBurstDatetime();
			$data->hostName = $svc->get_host_name();
			$data->serviceDescription = $svc->get_service_description();
			$data->contactName = $name;
			$data->contactAlias = $contact->get_alias();
			$data->email = $contact->get_email();
			$data->rDays = $rDays;
			$data->maxValue = $burst->getMaxValue();
			$data->unit = $burst->getUnit();
			$data->lang = $contact->get_language();
			$diff = strtotime($data->burstDatetime) - time();
			$remaining = $diff / 60 / 60 / 24;
			$unit = "dia";
			if ((int) $remaining === 0) {
				$remaining = $diff / 60 / 60;
				$unit = "hora";
			}
			if ((int) $remaining === 0) {
				$remaining = $diff / 60;
				$unit = "minuto";
			}
			$data->remaining = ((int) $remaining);
			$data->remainingUnit = $unit;

			foreach ($svc->get_metrics() as $metric) {
				if ($metric['metric_id'] == $burst->getMetricId()) {
					$data->metric = $metric['metric'];
					break;
				}
			}

			if (! $data->metric) {
				continue;
			}

			sendmail($data);
            sendONC($data, $contact);
            $count++;
		}
    }
}

if ($count > 0) {
    print date('Y-m-d H:i:s')." - Total of {$count} email(s) send.".PHP_EOL;
} else {
    print date('Y-m-d H:i:s')." - Don't have brust for send mail.".PHP_EOL;
}

function clean() {
    $table = Predict::getTable();
    $query = sprintf(
        "DELETE FROM %s WHERE last_check <= NOW() - INTERVAL 8 HOUR",
        $table
    );
    $db = new DefaultDB('opmon4');
    $db->prepare($query);
    $db->execute();
}
exit(0);

function sendONC($data, \Contact $contact)
{
    $remainingUnit = ($data->remaining > 1) ? $data->remainingUnit."s" : $data->remainingUnit;
    $text = "Neoson informa: detectado um possível incidente daqui a {$data->remaining} {$remainingUnit} no serviço {$data->serviceDescription} do host {$data->hostName}.";

    if ($data->lang === 'en_US') {
        $remainingUnitList = array(
            'dia', 'day',
            'hora', 'hour',
            'minuto', 'minute'
        );
        $remainingUnit = $remainingUnitList[$data->remainingUnit];
        $remainingUnit = ($data->remaining > 1) ? $data->remainingUnit."s" : $data->remainingUnit;
        $text = "Neoson informs: a possible incident in {$data->remaining} {$remainingUnit} has been detected on host {$data->hostName} and service {$data->serviceDescription}.";
    }

    $message = new StdClass();
    $message->icon = "ic_stat_4";
    $message->color = '#ff4a00';
    $message->text = $text;

    $user = ONCContact::get_contact_by_name($contact->get_contact_name());
    $token = new ONCToken();
    $token->setContactId($user->get_contact_id());
    $tokens = $token->getAllByContactId();

    try {
        $notify = ONCNotification::create_notification($message, $tokens);
        $response = ONCNotification::send_notification($notify);
    } catch (Exception $e) {
        print $e->getMessage().PHP_EOL;
    }
}

function sendmail($data)
{
    $subject = ($data->lang == 'en_US')
            ? "PREDICTION WARNING: Future capacity burst"
            : "ALERTA PREDITIVO: Futuro estouro de capacidade";

    /* Email Detials */
    $mail_to = $data->email;
    $from_mail = "OpMon";
    $from_name = ($data->lang == 'en_US')
         ? "OpMon Prediction"
         : "OpMon Preditivo";

    $message = getBodyEmail($data);

    /* Attachment File */
    // Attachment location
    $neoson = "neoson.png";
    $opmon = "logo_opmon.png";
    $path = "/usr/local/opmon/share/opcfg/images/";

    // Read the file content
    $file = $path.$neoson;
    $file_size = filesize($file);
    $handle = fopen($file, "r");
    $content = fread($handle, $file_size);
    fclose($handle);
    $contentNeoson = chunk_split(base64_encode($content));

    $file = $path.$opmon;
    $file_size = filesize($file);
    $handle = fopen($file, "r");
    $content = fread($handle, $file_size);
    fclose($handle);
    $contentOpmon = chunk_split(base64_encode($content));

    /* Set the email header */
    // Generate a boundary
    $boundary = md5(uniqid(time()));

    // Email header
    $header = "From: {$from_nam} < {$from_mail}>".PHP_EOL;
    $header .= "MIME-Version: 1.0".PHP_EOL;

    // Multipart wraps the Email Content and Attachment
    $header .= "Content-Type: multipart/mixed; boundary=\"{$boundary}\"".PHP_EOL;
    $header .= "This is a multi-part message in MIME format.".PHP_EOL;
    $header .= "--".$boundary.PHP_EOL;

    // Email content
    // Content-type can be text/plain or text/html
    $header .= "Content-type:text/html; charset='iso-8859-1'".PHP_EOL;
    $header .= "Content-Transfer-Encoding: 7bit".PHP_EOL.PHP_EOL;
    $header .= "$message".PHP_EOL;
    $header .= "--".$boundary.PHP_EOL;

    // Attachment
    // Edit content type for different file extensions
    $header .= "Content-Type: image/png; name=\"{$neoson}\"".PHP_EOL;
    $header .= "Content-Transfer-Encoding: base64".PHP_EOL;
    $header .= "Content-Disposition: inline".PHP_EOL;
    $header .= "Content-ID: <{$neoson}>".PHP_EOL.PHP_EOL;
    $header .= $contentNeoson.PHP_EOL;
    $header .= "--".$boundary.PHP_EOL;
    $header .= "Content-Type: image/png; name=\"{$opmon}\"".PHP_EOL;
    $header .= "Content-Transfer-Encoding: base64".PHP_EOL;
    $header .= "Content-Disposition: inline".PHP_EOL;
    $header .= "Content-ID: <{$opmon}>".PHP_EOL.PHP_EOL;
    $header .= $contentOpmon.PHP_EOL;
    $header .= "--".$boundary."--";

    // Send email
    $mail_sent = mail($mail_to, $subject, "", $header);
    print $mail_sent ? "Mail sent to {$mail_to}" : "Mail failed to {$mail_to}";
    print PHP_EOL;
}

function getBodyEmail($data)
{
    $hostText = "Host";
    $serviceText = "Serviço";
    $metricText = "Métrica";
    $dateText = "Data estimada";
    $hiText = "Olá, eu sou o Neoson";
    $format = "d/m/Y H:i";
    $remainingUnit = ($data->remaining > 1) ? $data->remainingUnit."s" : $data->remainingUnit;
    $alertText = "Vim do futuro para avisar que detectei um possível <span style='color:#C52525; font-weight: bold;'>incidente daqui a {$data->remaining} {$remainingUnit}!</span>";
    $titleImage = "Este é o Neoson, assistente virtual do OpMon";
    $title = "Alerta preditivo!";

    if ($data->lang === 'en_US') {
        $format = "m/d/Y H:i";
        $serviceText = "Service";
        $metricText = "Metric";
        $dateText = "Estimated date";
        $hiText = "Hi, I'm Neoson";
        $remainingUnitList = array(
            'dia', 'day',
            'hora', 'hour',
            'minuto', 'minute'
        );
        $remainingUnit = $remainingUnitList[$data->remainingUnit];
        $remainingUnit = ($data->remaining > 1) ? $data->remainingUnit."s" : $data->remainingUnit;
        $alertText = "I came back from the future to warn that I detected a possible <span style='color:#C52525; font-weight: bold;'>incident in {$data->remaining} {$remainingUnit}!</span>";
        $titleImage = "This is Neoson, OpMon's virtual assistant";
        $title = "Predictive Alert!";
    }

    $body = "
        width: 800px;
        margin: auto;
        font-family: 'Helvetica';
        font-size: 20px;
        background-color: #5687c1;
        color: #1c1c1c;
        padding: 20px 0;
        border-radius: 5px;
    ";

    $hi = "
      font-size: 22px!important;
      color: #444;
      font-weight: bold;
    ";

    $border = "
      border-radius: 10px;
      background-color: #fff;
      padding: 25px;
      width: 500px;
    ";

    $c1 = "
      border-radius: 5px;
      background-color: #FFF;
      color: #444;
      font-weight: bold;
      width: 150px;
    ";

    $c1Div = "
        background-color: #fff;
        text-align: right;
    ";

    $td = "
      font-size: 17px;
      text-align: left;
      word-break: normal;
      color: #444;
      border-radius: 5px;
      vertical-align: top;
    ";

    $tdDiv = "
      border-radius: 5px;
      padding: 12px;
      margin: 3px 5px;
      background-color: #f7f7f7;
    ";

    $html = '
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>
  <head>
     <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
     <meta name="viewport" content="width=device-width;">
    <title>'.$title.'</title>
  </head>
  <body>
    <div style="'.$body.'" align="center">
    <img src="cid:logo_opmon.png" hspace="200px" width="125px" height="auto" style="margin-bottom:10px">
    <div style="'.$border.'">
      <div>
        <img src="cid:neoson.png" hspace="170px" width="200px" title="'.$titleImage.'">
        <br/><p style="'.$hi.'">'.$hiText.'</p>
        <span style="color:#444; padding: 0 20px; margin: 0 20px; display: block;">'.$alertText.'</span></p>
        <table style="'.$table.'" width="100%" style="padding: 18px;">
          <tr>
          <td style="'.$c1.$td.'"><div style="'.$tdDiv.$c1Div.'">'.$hostText.'</div></td>
          <td style="'.$td.'"><div style="'.$tdDiv.'">'.$data->hostName.'</div></td>
          </tr>
          <tr>
          <td style="'.$c1.$td.'"><div style="'.$tdDiv.$c1Div.'">'.$serviceText.'</div></td>
          <td style="'.$td.'"><div style="'.$tdDiv.'">'.$data->serviceDescription.'</div></td>
          </tr>
          <tr>
          <td style="'.$c1.$td.'"><div style="'.$tdDiv.$c1Div.'">'.$metricText.'</div></td>
          <td style="'.$td.'"><div style="'.$tdDiv.'">'.$data->metric.'</div></td>
          </tr>
          <tr>
          <td style="'.$c1.$td.'"><div style="'.$tdDiv.$c1Div.'">'.$dateText.'</div></td>
          <td style="'.$td.'"><div style="'.$tdDiv.'">'.date($format, strtotime($data->burstDatetime)).'</div></td>
          </tr>
        </table>
        </div>
      </div>
    </div>
  </body>
</html>
';

    return $html;
}
