#!/bin/bash
#set -x

lockfile="/var/run/centos-mirror.lock"
errorfile="/var/log/centos-mirror-error.log"
log=/var/log/centos-mirror.log
/bin/rm -f $errorfile

if [ -e $lockfile ]; then
	mail -s "Lockfile exists in centos-mirros ns3" alessandro.ren@opservices.com.br < $lockfile
	echo "Lockfile exists, bailing out"
	exit 0
fi

# Specify the location of rsync
rsync="/usr/bin/rsync -avSHP --delete --exclude "local*" --exclude "isos" "

# Specify the rsync host
mirror=mirrors.usc.edu::centos
#
#rsync://mirrors.usc.edu/centos/

# Specify the base location on the rsync host (no leading or ending slash(/))
#mbase=centos

# Specify the version(s) to mirror
# E.g. ver="5? < Will mirror version 5 only
# E.g. ver="4.6 5? < Will mirror versions 4.6 & 5 only
ver="5.7"

baselist="os updates"
local=/var/local/products/centos

# Specify the architecture to mirror
# E.g. archlist="i386? < Will mirror the i386 architecture only
# E.g. archlist="i386 x86_64? < Will mirror the i386 & x86_64 architectures only
archlist="i386 x86_64"

# Specify which sections of the CentOS to mirror
# E.g. baselist="os" < Will mirror the OS section
# E.g. baselist="os updates" $log
# Get in the right directory and set the umask to be group writable
cd $local
umask 002

# rsync  -avSHP --delete --exclude "local*" --exclude "isos" mirrors.usc.edu::centos/5.7/ /var/local/products/centos/5.7/

echo "Logging to "$log
echo "Starting mirror of CentOS Ver."$ver > $log
echo "Mirror "$mirror >> $log
echo >> $log
for ver in $ver
do
	for arch in $archlist
	do
		for base in $baselist
		do
			mkdir -p $local/$ver/$base/$arch/
			remote=$mirror/$ver/$base/$arch/
			echo "Mirroring  "$mirror/$ver/$base/$arch/ >> $log
			$rsync $remote $local/$ver/$base/$arch/ >> $log
			run_cmd="$rsync $remote $local/$ver/$base/$arch/"
			RETVAL=$?
			if [ $RETVAL -ne 0 ]; then
			    echo $run_cmd >> $errorfile
			fi
		done
	done
done

if [ -e $errorfile ]; then
	mail -s "Error in rsync centos mirror in ns3" alessandro.ren@opservices.com.br < $errorfile
fi

/bin/rm -f $lockfile

exit 0
