#!/usr/bin/perl 
#Author:
#        Otavio Honorio (otavio.honorio@opservices.com.br)
use strict;
use Getopt::Long;
use POSIX;
use File::Basename;
use Date::Manip;
# Setting environment
$ENV{"USER"}="opuser";
$ENV{"HOME"}="/home/opuser";
# Global variables
our $name = basename($0, ".pl");
our $path = "/usr/local/opmon/libexec";
our ($oHelp, $oVerbose, $oWarn, $oCrit, $perf);
#--------------------------------------------------------------------------------------
sub main {
	getoption();
	my $logbackup = "/var/log/opbackup.log";
	open (FILE,"$logbackup") or quit("Log not found",3);
	my $date = strftime('%d/%m/%Y', localtime);
	my @lines = <FILE>;
	my ($count, @today, $mgs) = 0;
	foreach (@lines) {
		if ($lines[$count] =~ /$date/) {
			push(@today, $lines[$count]); 
		}
		$count++;
	}
	
	my $verify = scalar(@today);
	$mgs .= "<br> ".$_  foreach(@today);
	quit ("Backup executed more than once today: $mgs") if($verify > 2); 
	my $start = $today[0];
	my $finish = $today[1];
	my ($startDate, $finishDate);
	quit ("Backup not started: ".$lines[$#lines],2) if(!$start); 
	quit ("Backup is not successful, last message: ".$start,2) if(!$finish); 

	if ($start =~ /([\d]{2})\/([\d]{2})\/([\d]{4}).+([\d]{2}):([\d]{2}):([\d]{2}).+INIT FULL BACKUP.+/) {
		$startDate = "$1/$2/$3 $4:$5:$6";
	} else {
		quit ("Error in backup generation, last message: ".$start,2);
	}

	my ($gen, $send);
	if ($finish =~ /([\d]{2})\/([\d]{2})\/([\d]{4}).+([\d]{2}):([\d]{2}):([\d]{2}).+OK - FULL Backup SQL completed \/ Generated in (.+) sec\. \/ Send in (.+) sec\. :/) {
		$finishDate = "$1/$2/$3 $4:$5:$6";
		$gen = $7;
		$send = $8;
	} else {
		quit ("Error sending backup, last message: ".$finish,2);
	}

	my $result = $send + $gen;
	my $code = metric($send);
	my $perf = "Send=$send;;;0; Generation=$gen;;;0; Total=$result;$oWarn;$oCrit;0;";
	quit("Backup successfully executed in $result sec. | $perf",$code);
}
#--------------------------------------------------------------------------------------
sub metric {
	my $value = shift;
	if ($value >= $oCrit) { return 2
	}elsif ($value >= $oWarn) { return 1
	}elsif ($value < $oWarn) { return 0
	}else{ quit("Unable to check.",3) }
}
#--------------------------------------------------------------------------------------
sub quit {
	my $mgs = shift;
	my $code = shift;
	print $mgs,"\n";
	exit($code);
}
#--------------------------------------------------------------------------------------
sub getoption  {
	Getopt::Long::Configure('bundling');
	GetOptions(
		'c|critical=f' => \$oCrit,
		'h|help' => \$oHelp,
		'v|verbose=i' => \$oVerbose,
		'w|warning=f' => \$oWarn,
        );
	if($oHelp){
		printUsage();
		exit(1);
	}
	if ((!$oWarn) or (!$oCrit)){
		printUsage();
		exit(1);
	}
}
#--------------------------------------------------------------------------------------
sub logger {
        my $msg = shift (@_);
        my $log = "$path/$name.log";
        my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
        $wday++;
        $yday++;
        $mon++;
        $year+=1900;
        $isdst++;
        open(LOG, ">>$log");
        printf LOG ("%02i/%02i/%i - %02i:%02i:%02i => %s\n",$mday,$mon,$year,$hour,$min,$sec,$msg);
        close(LOG);
}
#--------------------------------------------------------------------------------------
sub printUsage {
       print <<EOB
Usage: $name.pl [OPTION]...

-c, --critical
-h, --help
-w, --warning
-v, --verbose

EOB
}
#--------------------------------------------------------------------------------------
&main;
