#!/usr/bin/perl
use strict;
use JSON;
our $statsfile = "/tmp/svn-stats";

sub main {
	my $contact = shift;
	my $count_trunk = 0;
	my $count_bf = 0;
	my $count_unknown = 0;
	my $count_hours = 0;
	my $by_contact = 1;
	my $record;
	my $outline;

	if (!defined $contact) {
		print "Please provide a contact name as a command line parameter\n";
		return 3;
	}

	$by_contact = 0 if ($contact eq "bybranch");

	if (!-f $statsfile) {
		$outline = sprintf(
		    "(nostatsfile) %s has %d changes on branch and %d on trunk|branch=%d;;;; trunk=%d;;;; hours=%.2fhours;;;;\n",
		    $contact,
		    $count_bf,
		    $count_trunk,
		    $count_bf,
		    $count_trunk,
		    $count_hours
		);
		print $outline;
		return 0;
	}

	open(STATSFILE,"<$statsfile");
	while(<STATSFILE>) {
		$record = decode_json($_);
		next if ($by_contact && $record->{user} ne $contact);

		if (defined $record->{hours}) {
			$count_hours += $record->{hours};
		}

		if ($record->{branch} =~ m/.*trunk.*/) {
			$count_trunk += $record->{changes};
			next;
		}

		if ($record->{branch} =~ m/.*bf\d\d.*/) {
			$count_bf += $record->{changes};
			next;
		}

		$count_unknown += $record->{branch};
	}
	close(STATSFILE);

	$outline = sprintf(
	    "%s has %d changes on branch and %d on trunk|branch=%d;;;; trunk=%d;;;; hours=%.2fhours;;;;\n",
	    $contact,
	    $count_bf,
	    $count_trunk,
	    $count_bf,
	    $count_trunk,
	    $count_hours
	);

	print $outline;
	return 0;
}


exit(&main($ARGV[0]));
