#!/usr/bin/perl

use DBI;
use strict;

our $version = "1.0";

our $slave_address = "192.168.10.4";
our $log_file = "/var/log/clean-incrementals.log";
our $mysqldir = "/var/lib/mysql";
our $conf_file = "/usr/local/opmon/etc/db.php";
our %date = _gettime();

our ( $dbuser, $dbpass, $dbhost, @dblist );

sub _main {

   _logger("Binary Log Cleaner startig.");

   _read_conf();
   _clean_incrementals();

   _logger("Done");

}

sub _gettime {

   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
   $mon++;
   $year+=1900;

   my %self = (
      'wday'	=>	$wday,
      'year'	=>	$year,
      'mon'	=>	$mon,
      'mday'	=>	$mday,
      'hour'	=>	$hour,
      'min'	=>	$min,
      'sec'	=>	$sec
   );

   return %self;

}

sub _logger {

   my ($msg) = shift;

   my %date = _gettime();

   open(_LOG_, ">>$log_file");
   print _LOG_ "$date{year}/$date{mon}/$date{mday} $date{hour}:$date{min}:$date{sec} - $msg\n";
   close(_LOG_);


}

sub _clean_incrementals {

   my $dbh = DBI->connect ( "dbi:mysql:dbname=mysql;host=$slave_address", "$dbuser", "$dbpass");
   die "Cannot connect to database!\n" if ( !defined $dbh );

   my $sth = $dbh->prepare("show slave status");
   die "Cannot prepare statement: $DBI::errstr\n" if ( !defined $sth ); 

   $sth->execute;

   my $return = $sth->fetchrow_hashref;
   my $rows = $sth->rows;
   if ($rows == 0 ) {
         print "Remote server $slave_address is not configured as a slave server\n";
         exit(0);
   }
   my $file = $mysqldir . "/" . $return->{Relay_Master_Log_File};
   _logger("Slave using: $file");
   my @stat = stat( $file ); 

   opendir( DIR, $mysqldir );
   my @inc_files = grep { $_ =~ /^incremental.*$/ and "$mysqldir/$_" ne $file } readdir DIR ;
   closedir( DIR );

   foreach( sort(@inc_files) ) {
      my @stat_file = stat( "$mysqldir/$_" );
      if ($stat[9]>$stat_file[9]) {
         _logger("Cleaning $mysqldir/$_");
         unlink("$mysqldir/$_");
      }
   }


}

sub _read_conf {

   open( FILE, "$conf_file" );
   while(<FILE>) {
      $dbuser = $1 if (m/DBUSER\s*=\s*[\'\"]*([^\'\"]*)[\'\"]*;/);
      $dbpass = $1 if (m/DBPASS\s*=\s*[\'\"]*([^\'\"]*)[\'\"]*;/);
      $dbhost = $1 if (m/DBHOST\s*=\s*[\'\"]*([^\'\"]*)[\'\"]*;/);
   }
   close(FILE);

}

&_main;
