<?php
include "/usr/local/opmon/etc/config.php";

set_logged_user('opmonadmin');

$cmd_space="/usr/bin/du -h /var/lib/mysql/";
$space_before=shell_exec($cmd_space);

clean_hosts();

print "\n\nSpace before cleaning\n";
print($space_before);
print "\nSpace after cleaning\n";
$space_after=shell_exec($cmd_space);
print $space_after;

print "\nDone cleaning\n";
exit(0);

function clean_hosts()
{
	print "\nDropping tables for which host_ids no longer exist\n";
	/* Clean host_state_change and service_state_change tables per host_id */ 
	$tablesOpmon=array();
	$tablesOpmon=get_all_table_names_db_opmon4_host_id();
	foreach ($tablesOpmon as $table_name) {
		list($t1, $t2, $t3, $host_id) = explode("_", $table_name);
		if (isset($host_id)) {
			$obj=get_host($host_id);
			if ($obj==false) {
				print "Dropping table $table_name\n";
				$query = "drop table $table_name;";
				$result=do_query($query,DBOPMON);
				$result->Close();
			}
		}
	}
	
	/* Clean service_perf tables per host_id */ 
	$tablesOpmon=array();
	$tablesOpmon=get_all_table_names_db_opperf_host_id();
	foreach ($tablesOpmon as $table_name) {
		list($t1, $t2, $host_id) = explode("_", $table_name);
		if (isset($host_id)) {
			$obj=get_host($host_id);
			if ($obj==false) {
				print "Dropping table $table_name\n";
				$query = "drop table $table_name;";
				$result=do_query($query,DBPERF);
				$result->Close();
			}
		}
	}

	/* get all host_ids and service_ids */
	$host_ids=get_host_ids();
	$service_ids=get_service_ids();
	
	/* Clean tables that have host_id */
	$tablesOpmon=array();
	$tablesOpmon=get_all_table_names_db(DBOPMON);
	print "\nCleaning tables by host_id and service_id database DBOPMON\n";
	foreach ($tablesOpmon as $table_name) {
		if ($table_name == "eventguard_rules")
			continue;
		if (table_has_host_id(DBOPMON, $table_name)) {
			print "Cleaning host_id - $table_name\n";
			$query="DELETE FROM $table_name WHERE host_id NOT IN $host_ids;";
			$result=do_query($query,DBOPMON);
			$result->Close();
		}
		if (table_has_service_id(DBOPMON, $table_name)) {
			print "Cleaning service_id - $table_name\n";
			$query="DELETE FROM $table_name WHERE service_id NOT IN $service_ids;";
			$result=do_query($query,DBOPMON);
			$result->Close();
		}
	}
	
	print "\nCleaning performance tables DBPERF\n";
	clean_performance_tables();
	print "\nDONE Cleaning performance tables DBPERF\n";

        print "\nCleaning audit\n";
        $data_old=date('Y-m-d', strtotime('-8 week'));
        $query="DELETE FROM audit WHERE entry_time<'$data_old';";
        print "$query\n";
        $result=do_query($query,DBOPMON);
        $result->Close();

	print "\nCleaning opcfg history\n";
	$data_old=date('Y-m-d', strtotime('-8 week'));
	$query="DELETE FROM history WHERE ctime<'$data_old';";
	print "$query\n";
	$result=do_query($query,DBOPCFG);
	$result->Close();
	
	print "\nCleaning event_log\n";
	$date_old=date('Y-m-d', strtotime('-5 week'));
	$query=" delete from event_log where entry_time<'$date_old';";
	print "$query\n";
	$result=do_query($query,DBOPMON);
	$result->Close();
	
	optimize_crucial_table_in_opmon4();

	/* Optimize all tables with engine TokudDB*/
	optimize_tokudb_tables();

	/* Optimize all tables with engine TokudDB */
	optimize_innodb_tables();

}	

function get_all_table_names_into_database_with_engine($database, $engine){

	$sql = "
		SELECT 
			table_name
		FROM 
			information_schema.tables 
		WHERE 
			LCASE(TABLE_SCHEMA) = LCASE('{$database}') 
			AND LCASE(ENGINE) = LCASE('{$engine}')
	";


	$resultset = do_query($sql, 'information_schema');

	while(!$resultset->EOF) {
		$collection[] = array_shift($resultset->fields);
		$resultset->MoveNext();
	}

	return $collection; 

}

function optimize_tokudb_tables() {
	/* Optimize all tables with tokudb engine */
	print "\nOptimize all tables with tokudb engine\n";

	$databases = get_all_database_names();

	foreach($databases as $base){
		$tables = get_all_table_names_into_database_with_engine($base, 'tokudb');
		run_optimize_into_tables($base, $tables);
	}
}

function optimize_innodb_tables() {
	/* Optimize all tables with tokudb engine */
	print "\nOptimize all tables with innodbdb engine\n";

	$databases = get_all_database_names();

	foreach($databases as $base){
		$tables = get_all_table_names_into_database_with_engine($base, 'innodb');
		print "$base \n";
		run_optimize_into_tables($base, $tables);
	}
}

function optimize_crucial_table_in_opmon4(){
	/* Optimize some crucial tables in opmon4 database*/
	print "\nOptimize some crucial tables in opmon4 database\n";
	
	$db_to_optimize = "opmon4";
	$tablesOpmon=array();
	$tablesOpmon[]="host_status";
	$tablesOpmon[]="service_status";
	$tablesOpmon[]="service_contacts";
	$tablesOpmon[]="service_status";
	$tablesOpmon[]="hostgroup_membership";
	$tablesOpmon[]="servicegroup_membership";
	run_optimize_into_tables($db_to_optimize, $tablesOpmon);
}

function run_optimize_into_tables($db_to_optimize, $tablesOpmon){

	$optimization_done = false;
	foreach ($tablesOpmon as $table_name) {
		print "Optimize database ".$db_to_optimize." table $table_name... ";
		$query="OPTIMIZE TABLE $table_name;";
		$result=do_query($query,$db_to_optimize);
		$result->Close();
		print "done\n";
		$optimization_done = true;
	}
	
	return $optimization_done;
}

function table_has_host_id($dbname, $table)
{
	$query="desc $table;";
	$result = do_query($query,$dbname,OPDB_FETCH_OBJ);
	$has_host_id=false;
	while( !$result->EOF ) {
		$obj=$result->fields;
		if ($obj->Field == "host_id") {
			$has_host_id=true;
			break;
		}
		$result->MoveNext();
	}
	$result->Close();
	return($has_host_id);
}

function table_has_service_id($dbname, $table)
{
	$query="desc $table;";
	$result = do_query($query,$dbname,OPDB_FETCH_OBJ);
	$has_service_id=false;
	while( !$result->EOF ) {
		$obj=$result->fields;
		if ($obj->Field == "service_id") {
			$has_service_id=true;
			break;
		}
		$result->MoveNext();
	}
	$result->Close();
	return($has_service_id);
}

function get_host_ids()
{
	$query="select host_id from host_status;";
	$result = do_query($query,DBOPMON,OPDB_FETCH_OBJ);
	$first=false;
	$host_ids='';
	while( !$result->EOF ) {
		$obj=$result->fields;
		if ($first===false) {
			$host_ids="(".$obj->host_id;
			$first=true;
		}
		else {
			$host_ids=$host_ids.",".$obj->host_id;
		}
		$result->MoveNext();
	}
	$result->Close();
	if ($first===true) {
		$host_ids=$host_ids.")";
		$first=true;
	}
	return($host_ids);
}

function get_service_ids()
{
	$query="select distinct service_id from service_status;";
	$result = do_query($query,DBOPMON,OPDB_FETCH_OBJ);
	$first=false;
	$service_ids='';
	while( !$result->EOF ) {
		$obj=$result->fields;
		if ($first===false) {
			$service_ids="(".$obj->service_id;
			$first=true;
		}
		else {
			$service_ids=$service_ids.",".$obj->service_id;
		}
		$result->MoveNext();
	}
	$result->Close();
	if ($first===true) {
		$service_ids=$service_ids.")";
		$first=true;
	}
	return($service_ids);
}

function get_all_table_names_db($dbname)
{
	$query="show tables;";
	$result = do_query($query,$dbname);

	$tables = array();
	while( !$result->EOF ) {
		$tables[]=array_shift($result->fields);
		$result->MoveNext();
	}
	$result->Close();
	return($tables);
}

function get_all_table_names_db_opmon4_host_id()
{
	$query="show tables like 'host_state_change%';";
	$result = do_query($query,DBOPMON);

	$tables = array();
	while( !$result->EOF ) {
		//$obj=$result->fields;
		$tables[]=array_shift($result->fields);
		$result->MoveNext();
	}
	$result->Close();

	$query="show tables like 'service_state_change%';";
	$result = do_query($query,DBOPMON);
	while( !$result->EOF ) {
		//$obj=$result->fields;
		$tables[]=array_shift($result->fields);
		$result->MoveNext();
	}
	$result->Close();
	return($tables);
}

function get_all_table_names_db_opperf_host_id()
{
	$query="show tables like 'service_perf%';";
	$result = do_query($query,DBPERF);

	$tables = array();
	while( !$result->EOF ) {
		$tables[]=array_shift($result->fields);
		$result->MoveNext();
	}
	$result->Close();
	return($tables);
}

function check_table_exists($table_name, $dbname)
{
	$query="show tables;";
	$result = do_query($query,$dbname);

	$tables = array();
	while( !$result->EOF ) {
		$tables[]=array_shift($result->fields);
		$result->MoveNext();
	}
	$result->Close();
	$found=array_search($table_name, $tables);
	return($found);
}

function get_all_database_names(){
	$query="show databases;";
	$result = do_query($query, "mysql");

	while( !$result->EOF ) {
		$database=array_shift($result->fields);

		if ( ($database!='information_schema') && ($database!='test') && ($database!='mysql') )
			$databases[]=$database;

		$result->MoveNext();
	}
	$result->Close();
	return($databases);
}

function clean_performance_tables() {
	$query="select host_id, service_id, metric, metric_id from service_metric order by host_id,service_id;";
	$result = do_query($query,DBOPCFG,OPDB_FETCH_OBJ);
	while( !$result->EOF ) {
		$obj=$result->fields;
		$sql="SELECT * from service_status where host_id='".$obj->host_id."' AND service_id='".$obj->service_id."';";
		$result2 = do_query($sql,DBOPMON,OPDB_FETCH_OBJ);
		$perftable="service_perf_".$obj->host_id;
		$table_found=check_table_exists($perftable, DBPERF);
		/* Service not beeng monitored anymore */
		if ($result2->RecordCount()==0 && $table_found!==FALSE) {
			print "Cleaning service_perf_".$obj->host_id."...";
			$sql_del="DELETE FROM service_perf_".$obj->host_id." WHERE service_id='".$obj->service_id."' AND metric_id='".$obj->metric_id."';";
			//print "$sql_del\n";
			do_query($sql_del,DBPERF,OPDB_FETCH_OBJ);
			$sql_del="DELETE FROM service_metric WHERE host_id='".$obj->host_id."' AND service_id='".$obj->service_id."' AND metric_id='".$obj->metric_id."';";
			//print "$sql_del\n";
			do_query($sql_del,DBOPCFG,OPDB_FETCH_OBJ);
			print "done\n";
		}
		else {
			$result2->Close();
		}
		$result->MoveNext();
	}
	$result->Close();
}

?>
