#!/usr/bin/perl
use strict;
use DBI;

use constant WIOKFILE => "/usr/local/opmon/share/seagull/modules/opinterface/.1ST_STEP_COMPLETE";
use constant SCOKFILE => "/usr/local/opmon/share/seagull/modules/opinterface/.2ND_STEP_COMPLETE";
use constant INITFILE => "/usr/local/opmon/share/opcfg/nagios-init-script.php";
use constant TRUE => 1;
use constant FALSE => 0;
use constant OK => 0;
use constant ERROR => 1;
use constant TAR => "/bin/tar";
use constant MKDIR => "/bin/mkdir";
use constant CP => "/bin/cp";
use constant MV => "/bin/mv";
use constant CHOWN => "/bin/chown";
use constant TOUCH => "/bin/touch";
use constant PHP => "/usr/bin/php";

# globals
our $nagios_init_script = FALSE;
our $httpd_init_script = FALSE;
our $config_dir;
our $current_step = 1;
our $max_step = 7;

# global colors
our $red="\e[1;31m";
our $green="\e[1;32m";
our $normal="\e[0m";

sub main() {

	my $ret;

	$ret = verify_user();
	if ($ret != OK) {
		print("You must run this script as system administrator. Please login as root and try again.\n");
		return ERROR;
	}

	print_top();
	my_pause();

	if (!-f WIOKFILE) {
		print($red."Did you finish your web installation?\n".$normal);
		print("You need to finish it before run this script.\n");
		print("Please point your browser to http://<server ip address>/opmon\n");
		print("and finish the first installation step, then return here.\n");
		print("I wait.\n");
		return ERROR;
	}

	configure_sudoers();
	do_backup();
	run_export();
	print_top();
	
	print("As last step of your configuration, we need to change\n");
	print("the ownership of $config_dir, thus Apache's daemons can\n");
	print("write a new configuration before export and restart Nagios.\n");
	my_pause();

	system(CHOWN ." apache.apache $config_dir -R >/dev/null 2>&1");
	system(TOUCH ." ".SCOKFILE." >/dev/null 2>&1");

	print("It's done. Please refer to your web browser to run export.\n");
	print("Thanks for using OpReports for Nagios\n");
	return OK;

}


sub run_export() {

	my $export_return;
	my $export_command;

	print_top();
	print("NAGIOS CONFIGURATION EXPORT\n");
	print("Now we are going to export your Nagios configuration files\n");
	my_pause();

	$export_command = PHP." -q /usr/local/opmon/share/opcfg/export.php 'opmonadmin' 'localhost'";

	print("Exporting Nagios configuration files from OpCFG database\t\t");
	$export_return = `$export_command`;
	print("[".$green."  OK  ".$normal."]\n");
	my_pause();


}


sub print_top() {

	system("clear");
	print("\n");
	print($green."OOOOOOOOOOOOOO".$normal."\tWELCOME TO OPREPORTS CONFIGURATION UTILITY\n");
	print($green."OOOOOOOOOOOOOO".$normal."\tI'll guide you during the rest of configuration\n");
	print($green."OOOOOOO".$red."PPPPPPP\n".$normal);
	print($green."OOOOOOO".$red."PPPPPPP\n".$normal);
	print($green."OOOOOOO".$red."PPP".$green."OOOO\n".$normal);
	print($green."OOOOOOO".$red."PPP".$green."OOOO\n".$normal);
	print($green."OOOOOOO".$red."PPP".$green."OOOO".$normal."\tProgress: $current_step/$max_step\n");
	print("\n");

}

sub do_backup() {

	my %db_data;
	my $dbh;
	my $dsn;
	my $sth;
	my @row;
	my $ts;
	my $opt;

	%db_data = parse_db_data();

	$dsn = "dbi:mysql:database=opcfg:host=".$db_data{'DBHOST'}.":3306";
	$dbh = DBI->connect($dsn,$db_data{'DBUSER'},$db_data{'DBPASS'})
	       or die "Can't connect to database: $DBI::errstr\n";

	$sth = $dbh->prepare('SELECT config_dir FROM nagios_main')
	       or die "Couldn't prepare statement: " . $dbh->errstr;


	$sth->execute();
	($config_dir) = $sth->fetchrow_array();

	print_top();
	print("BACKUP YOUR ORIGINAL NAGIOS CONFIGURATION FILES:\n");

	print("Now we're going to make a backup copy of your Nagios\n");
	print("configuration files under $config_dir");
	my_pause();
	
	$ts = time();
	system(TAR." -czvf /root/nagios-config-$ts.tar.gz $config_dir --exclude=your-nagios-original-config 2>/dev/null >/dev/null");

	# Let's make another copy.
	system(MKDIR " -p /tmp/your-nagios-original-config >/dev/null 2>&1");
	system(CP ." -Rfp $config_dir /tmp/your-nagios-original-config/ >/dev/null 2>&1");
	system(MV ." /tmp/your-nagios-original-config $config_dir >/dev/null 2>&1");

	print("Creating backup of your Nagios Configuration Files:\t\t[".$green."  OK  ".$normal."]\n");
	print("Your Nagios Backup file: /root/nagios-config-$ts.tar.gz\n");
	print("Do you want to visualize backup content?[Y/n] ");
	$opt = read_user_input("y");

	if($opt eq 'y' || $opt eq 'Y') {
		system(TAR ." -tf /root/nagios-config-$ts.tar.gz | more ");
	} 

	my_pause();
	print_top();

}


sub configure_sudoers() {

	print("CONFIGURING SUDOERS:\n\n");

	print("To be able to export a new Nagios configuration\n");
	print("using OpReports Configuration Utility the Apache's user\n");
	print("must be able to restart Nagios and Apache core daemons. To achieve\n");
	print("this you must type the path to Nagios and Apache init scripts.\n\n");
	print("Nagios' init script [/etc/init.d/nagios]: ");
	$nagios_init_script = read_user_input("/etc/init.d/nagios");

	while(!-f $nagios_init_script) {
		print("Trying to find $nagios_init_script:\t\t [".$red."FAILED".$normal."]\n");
		print("Nagios' init script [$nagios_init_script]: ");
		$nagios_init_script = read_user_input($nagios_init_script);
	}
	configure_init_script($nagios_init_script);

	open(IFILE,">".INITFILE);
	print(IFILE "<?php\ndefine('NAGIOSINITSCRIPT','$nagios_init_script');\n?>");
	close(IFILE);

	print("\n");

	print("Apache's(httpd) init script [/etc/init.d/httpd]: ");
	$httpd_init_script = read_user_input("/etc/init.d/httpd");

	while(!-f $httpd_init_script) {
		print("Trying to find $httpd_init_script:\t\t [".$red."FAILED".$normal."]\n");
		print("Apache's(httpd) init script [$httpd_init_script]: ");
		$httpd_init_script = read_user_input($httpd_init_script);
	} 
	configure_init_script($httpd_init_script);
	print("\n");

	my_pause();
	print_top();

}

sub configure_init_script() {

	my $init = $_[0];
	my $found = FALSE;

	open(SUDOERS,"</etc/sudoers");
	foreach (<SUDOERS>) {

		if (m/^apache\tALL=\(ALL\)\tNOPASSWD: $init$/) {
			print("Sudoers actually configured for $init:\t\t[".$green."  OK  ".$normal."]\n");
			$found = TRUE;
		}

	}
	close(SUDOERS);

	if ($found == FALSE) {
		print("Configuring /etc/sudoers for $init:\t\t[".$green."  OK  ".$normal."]\n");
		open(SUDOERS,">>/etc/sudoers");
		print(SUDOERS "apache\tALL=(ALL)\tNOPASSWD: $init\n");
		close(SUDOERS);
	}

}


sub read_user_input() {
	my $default;
	my $input;

	$default = $_[0];

	$input = <STDIN>;
	chomp($input);

	if ($input eq "") {
		$input = $default;
	}

	return $input;
}

sub my_pause() {
	print("\nPress ".$green."ENTER".$normal." to continue...");
	<STDIN>;
	print("\n");
	$current_step++;
}

sub verify_user() {
	if ($< != 0) {
		return ERROR;
	} else {
		return OK;
	}
}

sub parse_db_data() {

	my %db_data;

	open(DBFILE,"</usr/local/opmon/etc/db.php");
	foreach(<DBFILE>) {
		if (m/\$(.+) *="(.*)".*;$/) {
			$db_data{$1} = $2;
		}
	}
	close(DBFILE);

	return %db_data;

}

exit(main());
