#!/usr/bin/perl

use DBI;
use strict;

our $version = "1.5";

our $log_file = "/var/log/backup.log";
our $mysqlfull = "/var/tmp/opmondb/mysql-backup.sql";
our $mysqldir = "/var/local/mysql/var";
our $logbindir = "/var/tmp/opmondb";
our $mysql = "/usr/bin/mysql";
our $mysqldump = "/usr/bin/mysqldump";
our $conf_file = "/usr/local/opmon/etc/db.php";
our %date = _gettime();

our ( $dbuser, $dbpass, $dbhost, @dblist );
our @ignoredDBs = ( "information_schema", "mysql", "backup", "test" );

# ----- Change Log Process ----- #
our $nohup = "/usr/bin/nohup";
our $php = "/usr/bin/php";
our $clean_db = "/usr/local/opmon/utils/clean_db_opmon4.php > /var/log/opmon-cleandb.log";
# ------------------------------ #

sub _main {
	
	_logger("MySQL backup starting.");
	
	# Fedora compatibility
	if (!-d $mysqldir){
	$mysqldir = "/var/lib/mysql";
	}
	
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
	# Clean an optimize database before dumping on the first day of the month
	if ($mday==1) {
		# Run CleanDB Process #
		_logger(" -> First of the month, running CleanDB Process");
		system("time $php -q $clean_db 2>/dev/null");
	}

	mkdir($logbindir);
	_read_conf();
	#_getbackupage();
	
	#_slave("stop");
	
	if (($date{'wday'} == 5) || ("$ARGV[0]" eq "full")) {
	  _logger("Running Full Backup...");
	  _backupfull();
	}else{
	  _logger("Running Incremental Backup...");
	  _backupincremental();
	}
	#_slave("start");
	
	_logger("Done");
	
	# Imports perfdata and SLA data that did't make to the DB
	my $run_cmd="/usr/local/opmon/utils/import_perf_sla_data.php";
	system($run_cmd);
	_logger("Really Done"); 
	
	_logger("Finally Done"); 
}

sub _gettime {

   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
   $mon++;
   $year+=1900;

   my %self = (
      'wday'	=>	$wday,
      'year'	=>	$year,
      'mon'	=>	$mon,
      'mday'	=>	$mday,
      'hour'	=>	$hour,
      'min'	=>	$min,
      'sec'	=>	$sec
   );

   return %self;

}

sub _logger {

   my ($msg) = shift;

   my %date = _gettime();

   open(_LOG_, ">>$log_file");
   print _LOG_ "$date{year}/$date{mon}/$date{mday} $date{hour}:$date{min}:$date{sec} - $msg\n";
   close(_LOG_);


}

sub _backupfull {

   _logger(" * Rotating bin logs");
   `$mysql -h $dbhost -u $dbuser --password="$dbpass" <<SQL
flush logs;
exit
SQL`;

   _logger(" * Dumping databases:");

   # Get database backup list
   _get_databases_list();

   # Dump each databse in a diffrent file  
   foreach(sort(@dblist)) {
      _logger("  --> $_");
      system("$mysqldump -f -u $dbuser -h $dbhost --password=$dbpass --single-transaction --skip-add-drop-table $_ > $logbindir/$_.sql 2>/dev/null");
   }

   _logger(" * Cleaning old incrementals");
   opendir(_LOGBIN_, $logbindir);
   my @files = grep { $_ =~ /^incremental\.[0-9]+/ } readdir _LOGBIN_;
   closedir(_LOGBIN_);
   foreach my $file (@files) {
      _logger(" * * * $logbindir/$file deleted.");
      unlink("$logbindir/$file");
   }
   opendir(_MYSQL_, $mysqldir);
   my @files = grep { $_ =~ /^incremental\.[0-9]+/ and $_ ne &_getincremental() } readdir _MYSQL_;
   closedir(_MYSQL_);
   foreach my $file (@files) {
      _logger(" * * * $mysqldir/$file deleted.");
      unlink("$mysqldir/$file");
   }

}

sub _backupincremental {

   _logger(" * Rotating bin logs");
   `$mysql -h $dbhost -u $dbuser --password="$dbpass" <<SQL
flush logs;
exit
SQL`;
   
   _logger(" * Moving old bin logs to $logbindir");
   opendir(_MYSQL_, $mysqldir);
   my @files = grep { $_ =~ /^incremental\.[0-9]+/ and $_ ne &_getincremental() } readdir _MYSQL_;
   closedir(_MYSQL_);
   foreach my $file (@files) {
      _logger(" * * * $mysqldir/$file moved to $logbindir/$file");
      system("mv $mysqldir/$file $logbindir/$file");
   }

   
}

sub _getincremental {

   my $dbh = DBI->connect ( "dbi:mysql:dbname=mysql;host=$dbhost", "$dbuser", "$dbpass");
   die "Cannot connect to database!\n" if ( !defined $dbh );

   my $sth = $dbh->prepare("show master status");
   die "Cannot prepare statement: $DBI::errstr\n" if ( !defined $sth ); 

   $sth->execute;

   my ($incremental) = $sth->fetchrow();

   return "$incremental";

}

sub _getbackupage {

   my @stat = stat($mysqlfull);

   my $age = (((time() - $stat[9])/60)/60)/24;

   if ($age>7) {
      $ARGV[0] = "full";
   }

   # Remove old log file if is bigger then 10MB
   my @log_stat = stat($log_file);
   unlink($log_file) if ($log_stat[7] >= 10485760);

}

sub _read_conf {

   open( FILE, "$conf_file" );
   while(<FILE>) {
      $dbuser = $1 if (m/DBUSER\s*=\s*[\'\"]*([^\'\"]*)[\'\"]*;/);
      $dbpass = $1 if (m/DBPASS\s*=\s*[\'\"]*([^\'\"]*)[\'\"]*;/);
      $dbhost = $1 if (m/DBHOST\s*=\s*[\'\"]*([^\'\"]*)[\'\"]*;/);
   }
   close(FILE);

}

sub _get_databases_list {

   my @list = `$mysql -u $dbuser -h $dbhost --password=$dbpass <<EOF | grep -v Database
show databases;
quit
EOF`;

   foreach(@list) {
      chomp;
      push @dblist, $_ if (!_in_array($_));
   }

}

sub _in_array {

   my $value = shift;

   foreach(@ignoredDBs) {
      return 1 if m/$value/;
   }

   return 0;

}

sub _slave {

   my $param = shift;
   _logger(" * Slave $param");
   `$mysql -h $dbhost -u $dbuser --password="$dbpass" <<SQL
slave $param;
exit
SQL`;

}

&_main;
