<?php
include "/usr/local/opmon/etc/config.php";
set_logged_user("opmonadmin");
$mod = "EXTERNAL";

function read_f($f) {
	$content = file_get_contents($f);
	return json_decode($content);
}

function search_on_nodes($host_ips) {
	if (!is_array($host_ips))
		return false;
	
	$i_host_ips = array_map(
	    ip2long,
	    $host_ips
	);

	$q = "SELECT name ".
	     "FROM nodes ".
	     "WHERE nodip IN (%s)";

	$q = sprintf(
	     $q,
	     implode(",", $i_host_ips)
	);

	$ret = do_query($q, "nedi", OPDB_FETCH_OBJ);
	if ($ret->RecordCount() == 0) {
		$ret->Close();
		return false;
	}

	$name = $ret->fields->nade;
	$ret->Close();
	return $name;
}

function search_on_devices($host_ips) {
	if (!is_array($host_ips))
		return false;
	
	$i_host_ips = array_map(
	    ip2long,
	    $host_ips
	);

	$q = "SELECT device ".
	     "FROM devices ".
	     "WHERE devip IN (%s)";

	$q = sprintf(
	     $q,
	     implode(",", $i_host_ips)
	);

	$ret = do_query($q, "nedi", OPDB_FETCH_OBJ);
	if ($ret->RecordCount() == 0) {
		$ret->Close();
		return false;
	}

	$name = $ret->fields->device;
	$ret->Close();
	return $name;
}

function get_devname_by_ips($host_ips) {
	$name = search_on_devices($host_ips);
	if ($name !== false && !empty($name))
		return $name;

	$name = search_on_nodes($host_ips);
	if ($name === false || empty($name))
		return false;

	return $name;
}

function get_devip_by_name($name) {
	$q = "SELECT devip ".
	     "FROM devices ".
	     "WHERE device = '%s'";

	$q = sprintf(
	     $q,
	     do_escape_string($name)
	);

	$ret = do_query($q, "nedi", OPDB_FETCH_OBJ);
	if ($ret->RecordCount() > 0) {
		$ip = long2ip($ret->fields->devip);
		$ret->Close();
		return $ip;
	}

	$q = "SELECT nodip ".
	     "FROM nodes ".
	     "WHERE name = '%s'";
	$q = sprintf(
	     $q,
	     do_escape_string($name)
	);

	$ret = do_query($q, "nedi", OPDB_FETCH_OBJ);
	if ($ret->RecordCount() == 0)
		return false;

	$ip = long2ip($ret->fields->nodip);
	$ret->Close();
	return $ip;
}

function get_route($name) {
	global $devs;

	if (!isset($devs->$name))
		return false;

	return array_reverse(array_keys((array)$devs->$name->alk));
}


function get_host_ips_by_ip($ip) {
	$h = HostDiscovery::get_from_db_by_ips(array(
	    $ip
	));

	if ($h === false)
		return array($ip);
	
	return $h->get_ips();
}

function main() {
	$hosts = get_hosts();

	foreach($hosts as $h) {
		$host = get_host_by_id($h->host_id);

		$host_ips = get_host_ips_by_ip($host->get_address());
		$host_name_on_nedi = get_devname_by_ips($host_ips);
		if ($host_name_on_nedi === false) {
			//print "nedi doesnt known about: ";
			//print_r($host_ips);
			continue;
		}

		$route = get_route($host_name_on_nedi);
		if (is_null($route))
			continue;

		$parent_name = null;
		foreach($route as $hn) {

			$d = get_devip_by_name($hn);
			if ($d === false)
				continue;

			$dhost = HostDiscovery::get_from_db_by_ips(array(
			    $d
			));

			if ($dhost === false)
				continue;

			$parent_name = $dhost->get_host_name();
			break;
		}


		if (is_null($parent_name))
			continue;

		$child_id = get_host_id_by_name($host->get_host_name());
		$parent_id = get_host_id_by_name($dhost->get_host_name());
		if ($child_id === false || $parent_id === false)
			continue;

		$q = "UPDATE nagios_hosts SET parents=%d WHERE host_id=%d";
		$q = sprintf($q, $parent_id, $child_id);

		$ret = do_query($q, DBOPCFG);
		$ret->Close();
	}
}

$devs = read_f("alldevs.json");
main();

?>
