#!/usr/bin/php
<?php

include "/usr/local/opmon/etc/config.php";

$options = getopt("fh");

if (isset($options["h"])) {
	print "\nUsage: opmon-export.php [-f|-h]\n\n";
	print "\tScript to export opmon configuration\n";
	print "\tThis script will export only if there's a worker\n";
	print "\t'startExport' registered.If you want to export \n";
	print "\twithout check for 'startExport' use -f option\n";
	print "\tto force.\n";
	print "\n";
	exit(0);

}

$tr = new TaskRunner();
if (!isset($options["f"])) {
	$registered_workers = false;
	if (is_object($tr)) {
		$registered_workers = $tr->getRegisteredWorkers();
	}
	if ( !isset($registered_workers['startExport']) ||
	     $registered_workers['startExport'] != 1       ) {
		print "\nERROR: Unable to find startExport worker.\n";
		print "       If you want to force an export, use -f option\n\n";
		if (is_object($tr)) {
			$tr->disconnect();
		}
		exit;

	}
} else {
	print "WARNING: Forced export running.\n";
}

$opcfg_dir = "/usr/local/opmon/share/opcfg";
$log_file = $opcfg_dir . "/logs/export.log";
$lock_file = $opcfg_dir . "/export.lock";

$export_command = "/usr/bin/nohup /usr/bin/php -q $opcfg_dir/tools/exporter/export.php 1 opmonadmin console>/dev/null 2>/dev/null &";

if (is_object($tr) && $tr->testServerConnection() === true) {

	print "Starting export through gearmand\n";
	if ($tr->run('fileExists',$lock_file)) {
		print "Another export process is running.\n";
	} else {

		$obj = new stdClass();
		$obj->command = $export_command;
		$obj->log_file = $log_file;
		$str = json_encode($obj);
		$tr->run('startExport',$str);
		print "Export process started successfully.\n";

		sleep(1);

		print "Exporting: [\]";
		$query = "select is_running from export_log";
		$ret = do_query($query,DBOPCFG,OPDB_FETCH_OBJ);
		$count = 0;
		while($ret->fields->is_running) {

			if ($count == 4)
				$count = 0;

			give_feedback($count);
			$ret->Close();
			$ret = do_query($query,DBOPCFG,OPDB_FETCH_OBJ);
			$count++;
			sleep(1);

		}
		$ret->Close();

		print "\nFinished. Log:\n";

		$query = "select * from export_log";
		$ret = do_query($query,DBOPCFG,OPDB_FETCH_OBJ);
		while(!$ret->EOF) {

			$arr = json_decode($ret->fields->data);
			foreach($arr as $o) {
				if (isset($o->time))
					print "[$o->time][$o->status]  $o->desc \n";
			}

			$ret->MoveNext();
		}
		$ret->Close();
	}

} else { // Export @ localhost 

	print "Unable to connect to gearmand. Starting export process @ localhost.\n";
	if (file_exists($lock_file)) {
		print "Another export process is running.\n";
	} else {

		system("/usr/bin/php -q /usr/local/opmon/share/opcfg/tools/exporter/export.php 1 opmonadmin console");

	}

}
if (is_object($tr)) {
	$tr->disconnect();
}
$tr = null;
exit(0);

function give_feedback($count) {

	print chr(8).chr(8).chr(8);
	switch($count) {

	case 0:
		print "[|]";
		break;

	case 1:
		print "[/]";
		break;

	case 2:
		print "[-]";
		break;

	case 3:
		print "[\\]";
		break;
	}
}


?>
