#!/usr/bin/perl
use strict;
use DBI;
use POSIX;

our($CFGFILE) = "/usr/local/opmon/etc/config.php";
our($ERRORFILE) = "/usr/local/opmon/var/query-errors.log";
our($OPMON_QUERIES_FILE) = "/usr/local/opmon/var/opmon.sql";
our($PERF_QUERIES_FILE) = "/usr/local/opmon/var/opperf.sql";

sub main
{
	my(%dbconf);
	my($dbh);
	my($sth);
	my($sql);
	my($tmpstr);

	if (!-f $OPMON_QUERIES_FILE && !-f $PERF_QUERIES_FILE) {
		print("OK - No failed queries found\n");
		exit;
	}

	get_db_access_data(\%dbconf);

	# opmon database queries
	if (-f $OPMON_QUERIES_FILE) {

		print("Importing ".$OPMON_QUERIES_FILE." queries file\n");
		$tmpstr = "dbi:mysql:dbname=".$dbconf{"DBOPMON"}.";host=".$dbconf{"DBHOST"};

		$dbh = DBI->connect($tmpstr,$dbconf{"DBUSER"},$dbconf{"DBPASS"});
		if ( !defined $dbh ) {
        		log_error("Unable to connect database");
			exit;
		}

		open(FILE,"<$OPMON_QUERIES_FILE");
		foreach(<FILE>) {
			chomp($_);
			if ($_ =~ m/[a-z]+/) {
				exec_query(\$dbh,$_);
			}
		}
		close(FILE);

		
		open(FILE,">$OPMON_QUERIES_FILE");
		close(FILE);
	}

	# perfparse database queries
	if (-f $PERF_QUERIES_FILE) {

		print("Importing ".$PERF_QUERIES_FILE." queries file\n");
		$tmpstr = "dbi:mysql:dbname=".$dbconf{"DBPERF"}.";host=".$dbconf{"DBHOST"};

		$dbh = DBI->connect($tmpstr,$dbconf{"DBUSER"},$dbconf{"DBPASS"});
		if ( !defined $dbh ) {
        		log_error("Unable to connect database");
			exit;
		}

		open(FILE,"<$PERF_QUERIES_FILE");
		foreach(<FILE>) {
			chomp($_);
			if ($_ =~ m/[a-z]+/) {
				exec_query(\$dbh,$_);
			}
		}
		close(FILE);

		open(FILE,">$PERF_QUERIES_FILE");
		close(FILE);

	}

	print("Ending execution sucessfully\n");

}

sub exec_query
{
	my($dbh) = shift;
	my($sql) = shift;
	my($sth);
	my($ret);

	print("Executing query: $sql\n");

	$sth = $$dbh->prepare($sql);
	if ( !defined $sth ) {
		log_error("Error executing query...");
		log_error($sql);
	} else {
		$ret = $sth->execute;
		if (!$ret) {
			log_error("Failed Query:".$sql);
		}
	}
}

# parses database configuration file
sub get_db_access_data
{
	my($conf) = shift;

	print("Parsing database information...\n");

	if (!-f $CFGFILE) {
		log_error("Unable to find configuration file $CFGFILE");
		exit;
	}

	open(FILE,"<$CFGFILE");
	foreach(<FILE>) {
		if (m/^define\("(.*)"\b*,\b*"(.*)".*$/) {
			$$conf{$1} = $2;
		}
	}
	close(FILE);
}

sub log_error
{
	my($msg) = shift;
	my($ts) = time();

	open(EFILE,">>$ERRORFILE");
	print "[$ts] $msg\n";
	print EFILE "[$ts] $msg\n";
	close(EFILE);

}

main();
