#!/usr/bin/php
<?php

require("/usr/local/opmon/etc/config.php");

set_logged_user("opmonadmin");

CONST OPMONAPI = "http://%s/opmon/seagull/www/index.php/wsconnector/action/%s";

$opt = getOpts();

$sendCheckOpMonUrl = sprintf(OPMONAPI, $opt->{"opmon-address"}, "sendcheck");
$basicAuthOpMon = $opt->{"opmon-username"}.":".$opt->{"opmon-password"};

$conn = Livestatus\Connection\ConnectionProvider::factory("tcp");
$query = $conn->from("services");
$query->select(array("host_name", "description"));

$filter = array();
if ($opt->hostgroup) {
    $filters[] = array(
        "host_groups",
        Livestatus\LQL\Factory::createOperator(">="),
        $opt->hostgroup
    );
}

if ($opt->service) {
    $filters[] = array(
        "description",
        Livestatus\LQL\Factory::createOperator("~~"),
        $opt->service
    );
}

if (count($filters) === 1) {
    $query->filterBy($conn->createFilter($filters[0]));
} else if (count($filters) > 1) {
    $query->filterBy(
        $conn->createFilter(
            $filters,
            $conn->createOperator('And')
        )
    );
}

foreach ($conn->execute()->fetchAll() as $obj) {
    $sendCheckData = makePassiveCheck(
        $obj->host_name,
        $obj->description,
        $opt->message,
        "",
        $opt->status
    );

    sendPassiveCheck(
        $sendCheckOpMonUrl,
        $basicAuthOpMon,
        $sendCheckData
    );
}

exitOutput("Checagens executadas em: ".date("d/m/Y H:i:s").PHP_EOL, 0);

function help()
{
  print "
        Command line usage:
            ./sendPassiveChecks.php

        --hostgroup
        --service
        --message
        --status
        --opmon-address
        --opmon-username
        --opmon-password
        --help          Show help
  ";
  exit(0);
}

function getOpts()
{
  $longopts  = array(
	"hostgroup:",
	"service:",
	"message:",
	"status:",
	"opmon-address:",
    "opmon-username:",
    "opmon-password:",
    "verbose",
    "help"
  );

  $options = getopt("", $longopts);
  $keys = array_keys($options);
  if (empty($options)) {
    help();
  }

  if (! in_array('status', $keys)) {
    help();
  }
  if (! in_array('message', $keys)) {
    help();
  }
  if (in_array('help', $keys)) {
    help();
  }
  if (in_array('verbose', $keys)) {
    $options["verbose"] = true;
  }

  return (object) $options;
}

function doCurl($url, $header = null, $basicAuth = null, $post = null)
{
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL, $url);
    if ($header) {
		curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
	}
    if ($post) {
        curl_setopt($ch, CURLOPT_POST, true);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
	}
    if ($basicAuth) {
        curl_setopt($ch, CURLOPT_USERPWD, $basicAuth);
        curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
    }
	curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 60);
	curl_setopt($ch, CURLOPT_TIMEOUT, 60);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    //curl_setopt($ch, CURLOPT_VERBOSE, true);
	$result = curl_exec($ch);
    $code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
    curl_close($ch);

    $data = json_decode($result);
    if ($code !== 200) {
        switch ($code) {
            case 429:
                throw new Exception("429 - too many requests");
            default:
                throw new Exception("Unexpected HTTP code: {$code}");
        }
    }

	if ($data->errors || ! $data || ! $result) {
		throw new Exception("Err - request without data or json failed: {$url}");
	}

	return $data;
}

function exitOutput($message, $code)
{
  print $message.PHP_EOL;
  exit($code);
}

function makePassiveCheck($hostname, $servicedesc, $output, $perfdata, $state)
{
    $data = [
        "host_name" => $hostname,
        "service_description" => $servicedesc,
        "plugin_output" => $output,
        "perf_data" => $perfdata,
        "state" => $state
    ];

	return "jsonData=".json_encode($data);
}

function sendPassiveCheck($sendCheckOpMonUrl, $basicAuthOpMon, $sendCheckData)
{
    $data = doCurl(
        $sendCheckOpMonUrl,
        array('Content-Type: application/x-www-form-urlencoded'),
        $basicAuthOpMon,
        $sendCheckData
    );

    if ($data->success != true) {
        exitOutput(
            "Error for send check. {$data->message}",
            3
        );
    }
}
# set smartindent tabstop=2 shiftwidth=2 softtabstop=2 expandtab paste
