#!/usr/bin/perl 
use strict;
use Getopt::Long;
use POSIX;
use File::Basename;
use Net::FTP;
use Filesys::SmbClientParser;
use Switch;
use Net::SCP::Expect;
use Time::Out qw(timeout);
use Time::HiRes qw(gettimeofday tv_interval);
# Setting environment
$ENV{"USER"}="root";
$ENV{"HOME"}="/root";
# Global variables
our $name = basename($0);
our ($oHelp, $oVerbose, $oHost, $oUser, $oPass, $oDomain, $oMode, $oDirectory, $oSource, $tGen, $tSend, $oPort, $oShare);
#--------------------------------------------------------------------------------------
sub main {
	getoption();
	verify_dump();
	my ($fullpath, $filebkp) = gen_package();
	switch ($oMode) {
		case "scp" { scp($fullpath); }
		case "ftp" { ftp($fullpath,$filebkp); }
		case "smb" { smb($fullpath,$filebkp); }
		else { logger("ERROR - Unknown mode. Use ftp or smb."); }
	}
}
#--------------------------------------------------------------------------------------
sub smb {
	my $fullpath = shift;
	my $filebkp = shift;
        my $t0 = [gettimeofday];
	logger("Please define parameter Share") if (!$oShare);
        if ($fullpath) {
		my $smb = new Filesys::SmbClientParser (undef,(user => $oUser, password => $oPass, workgroup => $oDomain))
                 or do { my $remove = `/bin/rm -rf $fullpath`; logger("ERROR SMB - Cannot connect to $oHost"); exit; };

                $smb ->Host($oHost)
                 or do { my $remove = `/bin/rm -rf $fullpath`; logger("ERROR SMB - Cannot access to host $oHost"); exit; };

		$smb ->Share($oShare)
                 or do { my $remove = `/bin/rm -rf $fullpath`; logger("ERROR SMB - Cannot access to share $oDirectory"); exit; };

		$filebkp = $oDirectory."\\".$filebkp if ($oDirectory);
		$smb ->put($fullpath,$filebkp)
                 or do { print Dumper $smb; my $remove = `/bin/rm -rf $fullpath`; logger("ERROR SMB - Cannot put $fullpath"); exit; };
	
		my $remove = `/bin/rm -rf $fullpath`;
		my $finishdate = localtime();
		$tSend = tv_interval($t0);
		$tGen = sprintf('%0.3f',$tGen);
		$tSend = sprintf('%0.3f',$tSend);
		logger("OK - FULL Backup SQL completed / Generated in $tGen sec. / Send in $tSend sec.: $finishdate");
                exit;
	}else{
                logger("ERROR SMB - Can't compact files");
                exit;
        }
	logger("ERROR SMB - Unexpected return");
	exit;
}
#--------------------------------------------------------------------------------------
sub ftp {
	my $fullpath = shift;
	my $filebkp = shift;
        my $t0 = [gettimeofday];
        if ($fullpath) {
        	my $ftp = Net::FTP->new($oHost, Debug => 0)
                 or do { my $remove = `/bin/rm -rf $fullpath`; logger("ERROR FTP - Cannot connect to $oHost"); exit; };

                $ftp ->login($oUser,$oPass)
                 or do { my $remove = `/bin/rm -rf $fullpath`; logger("ERROR FTP - Cannot login"); exit; };

		$filebkp = $oDirectory."/".$filebkp if ($oDirectory);
                $ftp ->put($fullpath, $filebkp)
                 or do { my $remove = `/bin/rm -rf $fullpath`; logger("ERROR FTP - Cannot put $fullpath"); exit; };
                
		$ftp ->quit;
		my $remove = `/bin/rm -rf $fullpath`;
                my $finishdate = localtime();
		$tSend = tv_interval($t0);
		$tGen = sprintf('%0.3f',$tGen);
		$tSend = sprintf('%0.3f',$tSend);
                logger("OK - FULL Backup SQL completed / Generated in $tGen sec. / Send in $tSend sec.: $finishdate");
                exit;
	}else{
                logger("ERROR FTP - Can't compact files");
                exit;
        }
	logger("ERROR FTP - Unexpected return");
	exit;
}
#--------------------------------------------------------------------------------------
sub scp {
	my $fullpath = shift;
        my $t0 = [gettimeofday];
	logger("Please define parameter Share") if (!$oDirectory);
        if ($fullpath) {
		my $scp = Net::SCP::Expect->new(host=>$oHost, port=>$oPort, user=>$oUser, password=>$oPass)
                 or do { my $remove = `/bin/rm -rf $fullpath`; logger("ERROR SCP - Cannot login $fullpath"); exit; };
		$scp->scp($fullpath,$oDirectory)
                 or do { my $remove = `/bin/rm -rf $fullpath`; logger("ERROR SCP - Cannot put $fullpath"); exit; };

		my $remove = `/bin/rm -rf $fullpath`;
                my $finishdate = localtime();
		$tSend = tv_interval($t0);
		$tGen = sprintf('%0.3f',$tGen);
		$tSend = sprintf('%0.3f',$tSend);
                logger("OK - FULL Backup SQL completed / Generated in $tGen sec. / Send in $tSend sec. : $finishdate");
                exit;
	}else{
                logger("ERROR SCP - Can't compact files");
                exit;
        }
	logger("ERROR SCP - Unexpected return");
	exit;
}
#--------------------------------------------------------------------------------------
sub gen_package {	
	my $date = strftime('%m-%d-%Y',localtime);
	my $initdate = localtime();
	my $pathbackup = $oSource;
        my $filebkp = "full-$date\.tar\.bz2";
        my $fullpath = "/var/tmp/$filebkp";

        logger("--- INIT FULL BACKUP : $initdate ---");

	my @df = `/bin/df /var 2>/dev/null`;
	my @spaceleft = split(/ /,$df[1]);
	my $diskcheck = 0;
	if ($spaceleft[17] =~ m/(.+)%/){ $diskcheck = $1; }
	if ($diskcheck >= 99) { logger("ERROR - No space left on device: $df[1]"); exit; }
		
        opendir(DIR, $pathbackup) or do { logger("ERROR - Can't opendir $pathbackup : $!"); exit; };
        my @myfiles = grep (/.+/, readdir(DIR));
        closedir DIR;

        if (!@myfiles) { logger("ERROR - No files found in $pathbackup"); exit; }
	
	my $t0 = [gettimeofday];
        my $compact = `/bin/tar cvfj $fullpath $pathbackup/* 2>/dev/null`;
	$tGen = tv_interval($t0);

	return $fullpath, $filebkp;
}
#--------------------------------------------------------------------------------------
sub verify_dump {
	my $procs = `/bin/ps x | grep mysqldump | grep -v grep`;
	chomp($procs);
	if ($procs > 0) {
		logger("ERROR - MySQL Dump in execution.");
		exit;
	}	
}
#--------------------------------------------------------------------------------------
sub getoption {
     Getopt::Long::Configure('bundling');
     GetOptions(
		'h|help' => \$oHelp,
		'v|verbose' => \$oVerbose,
		'mode=s' => \$oMode,
		'directory=s' => \$oDirectory,
		'domain=s' => \$oDomain,
		'host=s' => \$oHost,
		'username=s' => \$oUser,
		'password=s' => \$oPass,
		'port=i' => \$oPort,
		'source=s' => \$oSource,
		'share=s' => \$oShare,
        );
	if ($oHelp) {
		printHelp();
		exit;
	}

	$oSource = "/var/tmp/opmondb" if (!$oSource);
	$oPort = 22 if (!$oPort);

	if ((!$oMode) or (!$oHost) or (!$oUser) or (!$oPass) or (!$oSource)) {
		print "Wrong Syntax. Use -h or --help\n";
		logger("ERROR - Wrong Syntax.");
		exit;
	}
}
#--------------------------------------------------------------------------------------
sub logger {
        my $msg = shift (@_);
        my $log = "/home/opuser/send_backup/opbackup.log";
        my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
        $wday++;
        $yday++;
        $mon++;
        $year+=1900;
        $isdst++;
	open(LOG, ">>$log");
	printf LOG ("%02i/%02i/%i - %02i:%02i:%02i => %s\n",$mday,$mon,$year,$hour,$min,$sec,$msg);
	close(LOG);
}
#--------------------------------------------------------------------------------------
sub printHelp {
	my $help = <<HELP;
-h, --help
-v, --verbose
--host
--username
--password
--port
	only mode SCP/SSH
--directory
	directory remote case for needed.
--source
	directory local for sent.
--share
	share on remote host.
--mode
	scp - send backup file for SCP/SSH (port 22 by default SSH).
	ftp - send backup file for FTP.
	smb - send backup file for SMBClient.
--domain
	Only mode smb.

	Example Syntax for FTP:
		$name.pl --host=192.168.10.168 --username=otto --password=rocket --mode=ftp --source=/var/tmp/opmondb --directory=backup

	Example Syntax for SMB:
		$name.pl --host=192.168.10.168 --username=otto --password=rocket --mode=smb --source=/var/tmp/opmondb --share=backup --directory=opmon
		Obs.: --directory=opmon\\\\test, for more directory (on Windows)
	
	Example Syntax for SCP
                $name.pl --host=192.168.10.168 --username=otto --password=rocket --mode=scp --source=/var/tmp/opmondb --directory=backup

HELP
	print $help;
}
#--------------------------------------------------------------------------------------
&main
