<?php

include("/usr/local/opmon/etc/config.php");

set_logged_user("opmonadmin");

use Database\Driver\DefaultDB as DB;
use Livestatus\Connection\ConnectionProvider as Livestatus;

$conn = Livestatus::factory("tcp");

$query = $conn->from("hosts");
$query->select(
    array(
        "HOST_ID",
        "name",
        "state",
        "plugin_output",
        "state_type",
        "last_state_change",
        "current_attempt"
    )
);

$hosts = $conn->executeWithAliases(
    array('host_id' => 'HOST_ID')
)->fetchAll();

foreach ($hosts as $host) {
    logger("Adding state change (".date("Y-m-d H:i:s", $host->last_state_change).") to host {$host->name} ({$host->host_id})");
    addStateChange($host);
}

$query = $conn->from("services");
$query->select(
    array(
        "HOST_ID",
        "SERVICE_ID",
        "host_name",
        "description",
        "state",
        "plugin_output",
        "state_type",
        "last_state_change",
        "current_attempt"
    )
);

$services = $conn->executeWithAliases(
    array(
        'host_id' => 'HOST_ID',
        'service_id' => 'SERVICE_ID'
    )
)->fetchAll();

foreach ($services as $service) {
    logger("Adding state change (".date("Y-m-d H:i:s", $service->last_state_change).") to service {$service->description} ({$service->service_id}) from host {$service->host_name} ({$service->host_id})");
    addStateChange($service);
}

function addStateChange($object)
{
    $db = new DB();

    $table = "opmon4.host_state_change_{$object->host_id}";
    $fields = array(
        ':entry_time' => date("Y-m-d H:i:s", $object->last_state_change),
        ':current_state' => $object->state,
        ':state_type' => $object->state_type,
        ':current_attempt' => $object->current_attempt,
        ':plugin_output' => $object->plugin_output
    );

    if ($object->service_id) {
        $fields[':service_id'] = $object->service_id;
        $table = "opmon4.service_state_change_{$object->host_id}";
    }

    $columns = implode(
        ', ',
        array_map(
            function ($item) {
                return str_replace(':', '', $item);
            },
            array_keys($fields)
        )
    );

    $values = implode(
        ', ',
        array_keys($fields)
    );

    $query = "REPLACE INTO {$table} ({$columns}) VALUES ({$values})";
    $db->prepare($query);
    $db->execute($fields);
}

function logger($msg)
{
    $msg = str_replace("\n", "", $msg);

    $date = new DateTime();
    $dateFormated = $date->format('Y-m-d H:i:s');

    $logFile = fopen("/var/log/setAvailabilityFromCurrentState.log", "a");

    $text = "[$dateFormated] $msg".PHP_EOL;
    fprintf($logFile, $text);
    fclose($logFile);
    print $text;
}
