#!/usr/bin/php
<?

require '/usr/local/opmon/etc/config.php';

set_logged_user("neoson");

$opt = getOpts();
$query = null;
$type = null;
if ($opt->host) {
    $query = "SELECT host_id from opcfg.nagios_hosts";
    $type = "host";
}
if ($opt->service) {
    $query = "SELECT host_id, service_id from opcfg.nagios_services WHERE host_id IS NOT NULL";
    $type = "service";
}

$byGroup = " {$type}_id IN (SELECT {$type}_id FROM opcfg.nagios_{$type}group_membership WHERE {$type}group_id IN (SELECT {$type}group_id FROM opcfg.nagios_{$type}groups WHERE {$type}group_name = '{$opt->group}'))";
if ($opt->where || $opt->group) {
    $query .= ($opt->host) ? " WHERE " : " AND ";
    if ($opt->group && ! $opt->where) {
        $query .= $byGroup;
    }
    if (! $opt->group && $opt->where) {
        $query .= $opt->where;
    }
    if ($opt->where && $opt->group) {
        $query .= "{$byGroup} AND {$opt->where}";
    }
}

$db = new \Database\Driver\DefaultDB();
$db->prepare($query);
$db->execute();
$objects = $db->fetchAll(\Database\Driver\DefaultDB::FETCH_OBJ);

foreach ($objects as $object) {
    $doc = new Documentation();
    $hostname = get_host_name_by_id($object->host_id);
    $description = "{$hostname}";
    if ($object->service_id) {
        $doc->set_service_id($object->service_id);
        $servicename = get_service_description_by_id(
            $object->host_id,
            $object->service_id
        );
        $description = "{$hostname} - {$servicename}";
    }
    if (objectHasDocumentation($object->host_id, $object->service_id)) {
        if (! $opt->{"force-update"}) {
            print "{$description} already has documentation.".PHP_EOL;
            continue;
        }
    }
    $doc->set_host_id($object->host_id);
    $doc->set_text(file_get_contents($opt->doc));
    $doc->set_next_revision(date("Y/m/d H:i:s"));
    $doc->set_last_revision(date("Y/m/d H:i:s", strtotime('+1 year')));
    $doc->set_is_complete(1);
    $doc->set_description($description);
    $doc->set_revision_interval_value(120);
    $doc->set_revision_interval_type("D");

    $data = $doc->store();
    print "{$description} documentation added.".PHP_EOL;
    var_dump($data);
}

function getOpts()
{
  $longopts  = array(
    "doc:",
    "where:",
    "group:",
    "host",
    "service",
    "force-update",
    "verbose",
    "help"
  );

  $options = getopt("", $longopts);
  $keys = array_keys($options);
  if (empty($options)) {
    help();
  }

  if (in_array('help', $keys)) {
    help();
  }
  if (! in_array('doc', $keys)) {
    help();
  }
  if (! in_array('host', $keys) && ! in_array('service', $keys)) {
    help();
  }
  if (in_array('host', $keys)) {
    $options["host"] = true;
  }
  if (in_array('service', $keys)) {
    $options["service"] = true;
  }
  if (in_array('force-update', $keys)) {
    $options["service"] = true;
  }
  if (in_array('verbose', $keys)) {
    $options["verbose"] = true;
  }

  return (object) $options;
}

function exitOutput($message, $code)
{
  print $message.PHP_EOL;
  exit($code);
}

function help()
{
  print "
        Command line usage:
            ./setDoc.php --doc /d/path/doc.txt --host --group Linux --where \"host_name like 'UNI%' and notifications_enabled = '1'\"

        --doc
        --where
        --group
        --host
        --service
        --force-update
        --help          Show help
  ";
  exit(0);
}
