#!/usr/bin/php
<?php

include("/usr/local/opmon/etc/config.php");
use Database\Driver\DefaultDB as DB;
set_logged_user("opmonadmin");

function logRequestError($soaph)
{
    $error = sprintf(
        "[%s]\nrequest header:\n%s\nrequest:\n%s\nresponse header:\n%s\nresponse:\n%s\n",
        time(),
        $soaph->__getLastRequestHeaders(),
        $soaph->__getLastRequest(),
        $soaph->__getLastResponseHeaders(),
        $soaph->__getLastResponse()

    );

    $fp = @fopen("/tmp/soap-errors.log", "a");
    fprintf($fp, "%s\n", $error);
    fclose($fp);
}

function getSoapHandler()
{
    global $url;
    $stream_context = stream_context_create(
        array(
            'http' => array(
                'protocol_version'  => 1.0
            )
        )
    );

    $client = new SoapClient(
        $url,
        array(
            "stream_context" => $stream_context,
            "trace" => 1
        )
    );

    return $client;
}

function logger($msg)
{
    $msg = str_replace("\n", "", $msg);

    $date = new DateTime();
    $dateFormated = $date->format('Y-m-d H:i:s');

    $logFile = fopen("/var/log/priax-topology.log", "a");

    $text = "[$dateFormated] $msg".PHP_EOL;
    fprintf($logFile, $text);
    fclose($logFile);
    print $text;
}

function runTraceroute($address)
{
    $traceroute = new TraceRouteIntegration();
    $traceroute->set_host_ip($address);
    //$traceroute->set_debug_on();
    return array_reverse($traceroute->run());
}

function getHostsFromPriax()
{
    $data = false;
    $soaph = getSoapHandler();
    try {
        $data = $soaph->__soapCall("getAttributesNodesByQuery",
            array(
                "getAttributesNodesByQuery" => array(
                    "queryNodeObject" => array(
                        "query" => 'exists (e.`BasicNetworkAddress\Hostname`) and (e.`OpMon\OpMonActive` = "Yes" and size(e.`OpMon\OpMonID`) > 0)',
                        "result" => 'e.`BasicNetworkAddress\ManageIPAddress`,e.`OpMon\OpMonID`,e.`OpMon\OpMonCIName`'
                    )
                )
            )
        );
    } catch (Exception $e) {
        print $e->getMessage().PHP_EOL;
        logRequestError($soaph);
    }

    return $data;
}

function getHostIdFromPriax($address)
{
    $data = false;
    $soaph = getSoapHandler();
    try {
        $data = $soaph->__soapCall("getHostByIp",
            array(
                "getHostByIp" => array(
                    "queryHostObject" => array(
                        "ip" => $address,
                        "resultNode" => array(
                            "attributes" => array(
                                "entry" => array(
                                    "key" => 'OpMon\OpMonID'
                                )
                            )
                        )
                    )
                )
            )
        );
    } catch (Exception $e) {
        print $e->getMessage().PHP_EOL;
        logRequestError($soaph);
    }

    return $data;
}

function getPropertie($node, $propertie)
{
    foreach ($node->attributes->entry as $properties) {
        if ($properties->key === $propertie) {
            return $properties->value;
        }
    }
}

function getHostIdByAddress($address)
{
    $db = new DB();
    $db->prepare(
        'SELECT host_id from opcfg.nagios_hosts where address = :address'
    );
    $db->execute(array(':address' => $address));
    return $db->fetch(DB::FETCH_OBJ)->host_id;
}

function ping($address)
{
    exec(
        "/bin/ping -n -w 1 -c 1 {$address}",
        $output,
        $code
    );
    return ($code == 0);
}

function getParents($host_id)
{
    $db = new DB();

    $db->prepare('
        SELECT parent_id FROM (
                SELECT parents AS parent_id
                FROM opcfg.nagios_hosts
                WHERE host_id = :host_id
            UNION
                SELECT parent_id
                FROM opcfg.nagios_host_parents
                WHERE child_id = :host_id ) AS t
        WHERE parent_id IS NOT NULL
        ');

    $db->execute(
        array(
            ':host_id' => $host_id
        )
    );

    return $db->fetchAll(DB::FETCH_COLUMN);
}

function getHostAlreadyMonitoredFromAddress($address)
{
    $hopNode = getHostIdFromPriax($address);
    $hopProps = get_object_vars($hopNode);
    $hopId = $hopNode->Node->attributes->entry->value;

    $hostObject = get_host($hopId);
    if ($hostObject == false) {
        $hopId = getHostIdByAddress($address);
    }

    return (is_numeric($hopId)) ? $hopId : null;
}

$priaxIntegration = Integration\IntegrationEngine::getOneEnabledByType(1);

if (! isset($priaxIntegration->setting->url)) {
    logger("Priax integration not configured.");
    exit;
}

$url = $priaxIntegration->setting->url;

logger("Starting topology");
$config = new ConfigManager();
$hosts = getHostsFromPriax();

$modified = false;
foreach ($hosts->Node as $node) {
    $id = getPropertie($node, 'OpMon\OpMonID');
    $name = getPropertie($node, 'OpMon\OpMonCIName');
    $address = getPropertie($node, 'BasicNetworkAddress\ManageIPAddress');
    $route = runTraceroute($address);
    if (empty($route)) {
        continue;
    }
    logger(PHP_EOL);
    logger("Run traceroute to host {$name} id({$id}) address({$address})");
    foreach ($route as $hop) {
        if (ping($hop['ip']) === false) {
            continue;
        }

        $hopId = getHostAlreadyMonitoredFromAddress($hop['ip']);
        if (is_null($hopId)) {
            logger("Create a new host: Hop-Interface {$hop['ip']}");
            $hopId = $config->addHost(
                array(
                    "host_name" => "Hop-Interface {$hop['ip']}",
                    "alias" => "Dynamic Topology Hop Interface IP",
                    "address" => $hop['ip']
                )
            );
            $config->associateContactGroupsToHost($hopId, array(1));
        } else {
            logger("Loading host id({$hopId}) with {$hop['ip']}");
        }

        $childParents = getParents($id);
        if (! empty($childParents)) {
            if (in_array($hopId, $childParents)) {
                logger("Skiping child {$id} ip({$address}) already father {$hopId} ip({$hop['ip']})");
                $address = $hop['ip'];
                $id = $hopId;
                continue;
            }
            $config->cfg->host_add_parent($id, $hopId);
            $modified = true;
            logger("Set aditional parent child {$id} ip({$address}) > father {$hopId} ip({$hop['ip']})");
            $address = $hop['ip'];
            $id = $hopId;
        } else {
            $config->modifyHost(
                array(
                    'host_id' => $id,
                    'parents' => $hopId
                )
            );
            $modified = true;
            logger("Set parent child {$id} ip({$address}) > father {$hopId} ip({$hop['ip']})");
            $address = $hop['ip'];
            $id = $hopId;
        }
    }
}

if ($modified === true) {
    $config->commit(1, true);
    logger("Finish applied changes");
    exit;
}

logger("Finish without changes");
exit;
