#!/usr/bin/php
<?php
include_once('/usr/local/opmon/etc/config.php');

$option = $argv[1];
$job = $argv[2];


function main($job, $option)
{
    if(empty($option)) {
        print("Option hasn't been provided!");
        Logger::log("StartWizardConfig", "Option hasn't been provided!");
    }

    if($option === "backup") {
        makeBackup($job);
    }


    if($option === "smtp") {
        smtp($job);
    }


    if($option === "network") {
        network($job);
    }


    if($option === "ntp") {
        ntp($job);
    }


    if($option === "revert") {
        rollback();
    }

}

function revert()
{
    $runner = new ShellCommandRunner();

    $runner->run("postconf -e relayhost=127.0.0.1");
    $runner->run("service postfix stop");
    $runner->run('chkconfig postfix off');

    copy('/root/backup.save', '/etc/cron.d/backup');
    $runner->run('service smb stop');
    $runner->run('chkconfig smb off');
    $runner->run('service crond restart');

    copy('/root/ifcfg-eth0.save', '/etc/sysconfig/network-scripts/ifcfg-eth0');
    $runner->run('service network restart');

    $runner->run("chkconfig --level 35 opmonstats off; service opmonstats stop");
}

function ntp($job)
{
    $job = json_decode($job, true);

    $runner = new ShellCommandRunner();
    $runner->run('service ntpd stop');
    $runner->run("ntpdate {$job['ntp_server']}");
    if ($runner->get_exit_code() != OK) {
        Logger::log("StartWizardConfig", $runner->get_stderr());
        return get_runtime_error($runner->get_stderr());
    }
    $file = '/etc/ntp.conf';
    $template = '/usr/local/opmon/share/seagull/modules/startwizard/utils/ntp.conf';
    $backup = '/root/ntp.conf.save';

    $content = file_get_contents($template);
    $content = str_replace('{{server}}', $job['ntp_server'], $content);

    file_exists($backup) or copy($file, $backup);

    if(!file_put_contents($file, $content)) {
        Logger::log("StartWizardConfig", "Unable to write content into ntp.conf");
    }

    $runner->run('chkconfig ntpd on');
    $runner->run('service ntpd stop');
    $runner->run("ntpdate {$job['ntp_server']}");

    $status = get_command_status($runner);

    if (isset($job['hwclock'])) {
        $runner->run('hwclock --hctosys');
    }

    $file = '/etc/localtime';
    $backup = '/root/localtime.save';

    file_exists($backup) or copy($file, $backup);

    $runner->run(sprintf(
        'ln -sf /usr/share/zoneinfo/%s/%s %s',
        $job['country'],
        $job['region'],
        $file
    ));

    $runner->run('service ntpd start');
    Logger::log("StartWizardConfig", $status['stdout']);
    return $status;
}

function network($job)
{
    $job = json_decode($job, true);
    $file = '/etc/sysconfig/network-scripts/ifcfg-eth0';
    $backup = '/root/ifcfg-eth0.save';
    $template = '/usr/local/opmon/share/seagull/modules/startwizard/utils/ifcfg';

    file_exists($backup) or copy($file, $backup);

    $content = file_get_contents($template);

    $data = "BOOTPROTO=dhcp";
    if (
        ! empty($job['ipaddr']) &&
        ! empty($job['netmask']) &&
        ! empty($job['gateway'])

    ) {
        $data = sprintf(
            "IPADDR=%s\nNETMASK=%s\nGATEWAY=%s",
            $job['ipaddr'],
            $job['netmask'],
            $job['gateway']

        );

    }

    $content = str_replace('{{IPADDR}}', $data, $content);

    $data = "";
    if (! empty($job['dns1'])) {
        $data = "DNS1=" . $job['dns1'];

    }

    if (! empty($job['dns1']) && ! empty($option['dns2'])) {
        $data .= "\nDNS2=" . $job['dns2'];

    }

    $content = str_replace('{{DNS}}', $data, $content);

	if(!file_put_contents($file, $content)) {
        Logger::log("StartWizardConfig", "Unable to write content into ifcfg-eth0");
        return get_runtime_error('Unable to write content into ifcfg-eth0');
    }
    $runner = new ShellCommandRunner();
    $runner->run("sed -i 's/HOSTNAME=.*/HOSTNAME={$job['hostname']}/' /etc/sysconfig/network");
    $runner->run("hostname {$job['hostname']}");
    $runner->run('service network restart');
    $is_conf_incorrect = (strpos($runner->get_stderr(), 'pcalc: bad') != false);

    if ($is_conf_incorrect) {
        Logger::log("StartWizardConfig", "Some network settings are invalid");
        return get_runtime_error('Some network settings are invalid');
    }
    $output = get_command_status($runner);
    Logger::log("StartWizardConfig", $output['stdout']);
    return get_command_status($runner);
}

function smtp($job)
{
    $job = json_decode($job, true);

    $runner = new ShellCommandRunner();
    $runner->run('chkconfig postfix on');
    $runner->run("postconf -e relayhost={$job['relayhost']}");
    $output = get_command_status($runner);
    Logger::log("StartWizardConfig", $output['stdout']);
    return get_command_status($runner);
}

function makeBackup($job)
{
    $job = json_decode($job, true);

    $file = '/etc/cron.d/backup';
    $backup = '/root/backup.save';
    $template = '/usr/local/opmon/share/seagull/modules/startwizard/utils/cron-backup';
    file_exists($backup) or copy($file, $backup);

    $content = file_get_contents($template);

    $frequency = array('* * *', '* * 7', '1 * *');
    $index = $job['backup_frequency'];
    $content = str_replace('{{frequency}}', $frequency[$index], $content);

    if(!file_put_contents($file, $content)) {
        Logger::log("StartWizardConfig","Unable to write content into backup");
        return get_runtime_error('Unable to write content into backup');
    }

    $runner = new ShellCommandRunner();
    $runner->run('service smb restart');
    $runner->run('chkconfig --level 345 smb on');
    $runner->run('service crond restart');

    $output = get_command_status($runner);
    Logger::log("StartWizardConfig", $output['stdout']);

    return get_command_status($runner);
}

function get_command_status($runner)
{
    return array(
        'exit_code' => $runner->get_exit_code(),
        'stdout' => $runner->get_stdout(),
        'stderr' => $runner->get_stderr()
    );
}

function get_runtime_error($error_message)
{
    return array(
        'exit_code' => CRITICAL,
        'stdout' => '',
        'stderr' => $error_message
    );
}

main($job, $option);
