#!/bin/sh

#
# This script syncronize OpMon configuration and
# log files between cluster's two nodes
#

log() {

	DATE=`/bin/date | /usr/bin/xargs echo -n`
	echo "[$DATE] $1" >> $LOGFILE

}

# Include configuration file
if [ ! -f "/etc/syncfiles.conf" ]
then
	echo "Unable to find /etc/syncfiles.conf..."
	exit 2
fi
source /etc/syncfiles.conf

if [ $HOST = "INVALIDHOST"  ]; then
	echo "You need to edit /etc/syncfiles.conf first... getting out."
	exit 2
fi

if [ -z $CHECKFOROPMON ]; then
	CHECKFOROPMON=1
fi

if [ -z $CHECKTCPCONN ]; then
	CHECKTCPCONN=1
fi

if [ -z $USER  ]; then
	log "Using default 'root' user. You should set another user in syncfiles.conf."
	USER=root
fi

log "Starting syncfiles..."
log "Connecting to $HOST:$PORT - PID $$"
echo $$ >$PIDFILE

# Calcule sleeptime in seconds
#SLEEPTIME=`expr $SLEEPTIME \* 60`

while [ 1 ]; do

	FAIL=0

	# do we need to check for OpMon daemon?
	if [ $CHECKFOROPMON -eq 1 ]; then

		OPMONPID=`/bin/cat $OPMONLOCKFILE 2>/dev/null`
		RUNNING=`/bin/ps -p $OPMONPID 2>/dev/null | grep opmon|wc -l`
		if [ $RUNNING != 1 ] || [ ! -f $OPMONLOCKFILE ]; then
			log "OpMon isn't running... We don't need sync"
			FAIL=1
		fi


	fi


	for H in $HOST
	do

		FAIL=0

		# Connectivity
		if [ $CHECKTCPCONN -eq 1 ]; then

			$CHECKTCP -H $H -p $PORT >/dev/null 2>&1
			if [ $? -ne 0 ]; then
				log "Unable to connect with server $H:$PORT... bailing out"
				FAIL=1
			fi

		fi

		log "Sincronizing $H:$PORT"

		#
		# Copy files to slacker side of the force 
		#

		if [ $FAIL -eq 0 ]
		then

			# Mount and run rsync command acording config files
			for i in `/bin/ls -1 $CFGFILES`; do

				TMPDIR=`/bin/cat $i |/bin/grep dir= | /usr/bin/head -1 |cut -d= -f2`
				TMPDIR=`/bin/echo -n $TMPDIR`

				TODIR=`/bin/cat $i |/bin/grep to= | /usr/bin/head -1 |cut -d= -f2`
				TODIR=`/bin/echo -n $TODIR`

				log "Syncronizing $TMPDIR to $TODIR"

				TMPEXCLUDE=""
				# Exclude subdirs and files
				for j in `/bin/cat $i |/bin/grep exclude= | cut -d= -f2`
				do
					TMPEXCLUDE=$TMPEXCLUDE" --exclude=$j"
					TMPEXCLUDE=`/bin/echo -n $TMPEXCLUDE`
				done

				# Run rsync command
				LOG=`$RSYNC -r -a $TMPEXCLUDE -v -l --delete -e "ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -p $PORT" $TMPDIR $USER@$H:$TODIR 2>/dev/null`

				if [ $? -ne 0 ]; then
					log "Critical error in last sync process... "
				else
					log "$LOG"
				fi
			done
		fi

	done

	log "Sleeping..."
	$SLEEP $SLEEPTIME

done

