<?php

include("/usr/local/opmon/etc/config.php");

use Database\Driver\DefaultDB;

global $DBUSER;
global $DBPASS;
global $DBHOST;

print date('d-m-Y H:i:s')." - Starting migration".PHP_EOL;

$cfgMgr = new ConfigManager();
$cfgMgr->dont_check_permissions();
$confChanged = false;
$scats = array();

$pdo = new \PDO(
    "mysql:host={$DBHOST};",
    $DBUSER,
    $DBPASS,
    array(
        \PDO::ATTR_PERSISTENT => true,
        \PDO::MYSQL_ATTR_INIT_COMMAND => "SET NAMES utf8"
    )
);

$db = new DefaultDB('mysql', $pdo);

$query = '
    SELECT
        scat_type_name,
        host_id
    FROM
        opcfg.scat_types
    WHERE
        host_id IS NOT null
';

$db->prepare($query);
$db->execute();
$types = $db->fetchAll(DefaultDB::FETCH_OBJ);

if (! empty($types)) {
    print date('d-m-Y H:i:s')." - Migrating hosts for original service catalog type name".PHP_EOL;
    foreach ($types as $type) {
        $query = '
            SELECT
                count(host_id) as exist
            FROM
                opcfg.nagios_hosts
            WHERE
                host_id = :host_id
        ';

        $db->prepare($query);
        $db->execute(array(':host_id' => $type->host_id));
        $row = $db->fetch(DefaultDB::FETCH_OBJ);

        if ($row->exist == 0) {
            print date('d-m-Y H:i:s')." - Creating host {$type->scat_type_name} with id {$type->host_id}".PHP_EOL;
            $query = '
                INSERT INTO opcfg.nagios_hosts VALUES (
                    :host_id,
                    1,
                    "IT",
                    "IT",
                    "localhost",
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    "1",
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    2,
                    2,
                    161,
                    NULL,
                    NULL,
                    NULL,
                    DATE_FORMAT(NOW(),"%Y-%m-%d %H:%i:%s"),
                    NULL
                )
            ';

            $db->prepare($query);
            $db->execute(array(':host_id' => $type->host_id));
        }

        print date('d-m-Y H:i:s')." - Updating host {$type->scat_type_name} ".
            "of host id: {$type->host_id}".PHP_EOL;

        $query = '
            UPDATE
                opcfg.nagios_hosts
            SET
                host_name = :host_name
            WHERE
                host_id = :host_id
            ';

        $db->prepare($query);
        $db->execute(
            array(
                ':host_id' => $type->host_id,
                ':host_name' => $type->scat_type_name
            )
        );
    }
    print date('d-m-Y H:i:s')." - Done.".PHP_EOL;

    print date('d-m-Y H:i:s')." - Migrating services for original services catalog name".PHP_EOL;
    $query = '
        SELECT
            service_id,
            parameter as scat_id
        FROM
            opcfg.nagios_services_check_command_parameters
        WHERE
            service_id IN (
                    SELECT
                        service_id
                    FROM
                        opcfg.nagios_services
                    WHERE host_id IN (
                        SELECT
                            host_id
                        FROM
                            opcfg.scat_types
                        WHERE
                            host_id IS NOT null
                    )
                )
        GROUP BY
                service_id
        HAVING
                COUNT(scat_id) = 1
    ';

    $db->prepare($query);
    $db->execute();
    $serviceCatalogsMonitored = $db->fetchAll(DefaultDB::FETCH_OBJ);

    foreach ($serviceCatalogsMonitored as $svc) {
        $scats[] = $svc->scat_id;
        print date('d-m-Y H:i:s')." - Updating service id {$svc->service_id} ".
            "of service catalog id: {$svc->scat_id}".PHP_EOL;

        $query = '
            SELECT
                scat_name
            FROM
                opmon4.scats
            WHERE
                id = :scat_id
        ';

        $db->prepare($query);
        $db->execute(
            array(
                ':scat_id' => $svc->scat_id
            )
        );
        $scatName = $db->fetch(DefaultDB::FETCH_COLUMN);

        if (! $scatName) {
            print date('d-m-Y H:i:s')." - Service catalog ({$svc->scat_id}) not found".PHP_EOL;
            continue;
        }

        $query = '
            UPDATE
                opcfg.nagios_services
            SET
                service_description = :scat_name
            WHERE
                service_id = :service_id
        ';

        $db->prepare($query);
        $db->execute(
            array(
                ':service_id' => $svc->service_id,
                ':scat_name' => $scatName
            )
        );
        $confChanged = true;
    }
    print date('d-m-Y H:i:s')." - Done.".PHP_EOL;
}

if (! empty($scats)) {

    print date('d-m-Y H:i:s')." - Verifying service catalogs to be monitored".PHP_EOL;
    $query = '
        SELECT
            *
        FROM
            opmon4.scats
        WHERE
            id NOT IN ('.implode(',', $scats).')
    ';

    $db->prepare($query);
    $db->execute();
    $scatsNotMonitored = $db->fetchAll(DefaultDB::FETCH_OBJ);

    $types = get_scat_types();

    foreach ($scatsNotMonitored as $scat) {
        $scatType = $types[$scat->type];

        if ($scatType->host_id == NULL) {
            $newConfig = array(
                "host_name" => changeCharCode($scatType->scat_type_name),
                "alias" => $scatType->scat_type_name,
                "use_template_id" => $scatType->host_template_id,
                "address" => "localhost",
                "active_checks_enabled" => "1"
            );

            $extInfo = array(
                'icon_image' => "computer2.png"
            );

            $scatType->host_id = $cfgMgr->addHost($newConfig, $extInfo);
            $cfgMgr->modifyScatType($scatType);
        }

        $serviceConfig = array(
            "host_id" => $scatType->host_id,
            "service_description" => changeCharCode($scat->scat_name),
            "check_command" => "opmon_check_scat_status",
            "use_template_id" => $scatType->service_template_id
        );

        $service_id = $cfgMgr->addService($serviceConfig);
        $cfgMgr->addServiceCheckParameter($service_id, $scat->id);
        $confChanged = true;

        print date('d-m-Y H:i:s')." - Adding monitoration of service catalog ({$scat->scat_name} on".
            " host id ($scatType->host_id))".PHP_EOL;
    }

    if ($confChanged === true) {
        print date('d-m-Y H:i:s')." - Export configuration.".PHP_EOL;
        if ($cfgMgr->is_auto_export_enabled() === true) {
            $cfgMgr->commit(1, true);
        }
    }
}
