#!/usr/bin/perl
use strict;
use File::Copy;

our($my_cnf) = "/etc/my.cnf";
our($php_ini) = "/etc/php.ini";
our($httpd_conf) = "/etc/httpd/conf/httpd.conf";

sub main {

	my $mysql_change=0;
	my $php_change=0;
	my $http_change=0;
	$mysql_change=change_mysql();
	$php_change=change_php();
	$http_change=change_http();
	if ($mysql_change==1) {
			print "Restarting mysqld...\n";
		`/sbin/service mysqld restart`;
	}
	
	if ($php_change==1 or $http_change==1) {
		print "Restarting httpd...\n";
		`/sbin/service httpd restart`;
	}
	print "Done\n";
}

sub change_http {
	my $tmp_file = "/tmp/changed-httpd.conf";
	my $httpd_conf_bkp = "/tmp/httpd.conf-BKP";
	my $need_restart=0;

	print "Patching $httpd_conf\n";
	copy($httpd_conf, $httpd_conf_bkp) or die("\tCannot create backup copy of $httpd_conf -> $httpd_conf_bkp");

	open(CFGFILE,"<$httpd_conf") or die("\tUnable to find $httpd_conf");
	open(OUTFILE,">$tmp_file") or die("\tUnable to open temp file $tmp_file");

	foreach (<CFGFILE>) {
		if (m/^AddDefaultCharset.+$/) {
			if (m/UTF.+$/) {
				print OUTFILE $_;
			}
			else {
				print OUTFILE "AddDefaultCharset UTF-8\n";
				$need_restart=1;
			}
		} else {
			print OUTFILE $_;
		}
	}
	
	copy($tmp_file, $httpd_conf) or die("\tCannot copy $tmp_file to $httpd_conf");
	
	close(OUTFILE);
	close(CFGFILE);
	print "\tDone\n";
	return $need_restart;

}

sub change_php {

	my $tmp_file = "/tmp/changed-php.ini";
	my $php_ini_bkp = "/tmp/php.ini-BKP";
	my $need_restart=0;

	print "Patching $php_ini\n";
	copy($php_ini, $php_ini_bkp) or die("\tCannot create backup copy of $php_ini -> $php_ini_bkp");

	open(CFGFILE,"<$php_ini") or die("\tUnable to find $php_ini");
	open(OUTFILE,">$tmp_file") or die("\tUnable to open temp file $tmp_file");

	foreach (<CFGFILE>) {
		if (m/^default_charset.+$/) {
			if (m/UTF.+$/) {
				print OUTFILE $_;
			}
			else {
				print OUTFILE "default_charset = \"UTF-8\"\n";
				$need_restart=1;
			}
		} else {
			print OUTFILE $_;
		}
	}
	
	copy($tmp_file, $php_ini) or die("\tCannot copy $tmp_file to $php_ini");
	
	close(OUTFILE);
	close(CFGFILE);
	print "\tDone\n";
	return $need_restart;
}

sub change_mysql {
	my $first;
	my $tmp_file = "/tmp/changed-my.cnf";
	my $my_cnf_bkp = "/tmp/my.cnf-BKP";

	print "Patching $my_cnf\n";
	copy($my_cnf, $my_cnf_bkp) or die("\tCannot create backup copy of $my_cnf -> $my_cnf_bkp");
	open(CFGFILE,"<$my_cnf") or die("\tUnable to find $my_cnf");

	open(OUTFILE,">$tmp_file") or die("\tUnable to open temp file $tmp_file");
	print OUTFILE "[mysql]\n";
	print OUTFILE "default-character-set=utf8\n";

	$first = 1;
	foreach (<CFGFILE>) {
		if ($first) {
			$first = 0;
			if ($_ =~ m/^\[mysql\]$/) {
				print "\t$my_cnf already patched\n";
				close(OUTFILE);
				close(CFGFILE);
				return 0;
			}
		}

		if (m/^character_set_server\s*=\s*latin1\s*$/) {
			print OUTFILE "character_set_server=utf8\n";
			print OUTFILE "default-collation=utf8_unicode_ci\n";
		} else {
			print OUTFILE $_;
		}
	}
	close(OUTFILE);
	close(CFGFILE);

	copy($tmp_file, $my_cnf) or die("\tCannot copy $tmp_file to $my_cnf");
	print "\tDone\n";
	return 1;
}

&main();
